// 
// SoapTrace.Net - A soap message monitoring utility. 
// 
// Copyright (C) 2004 Matthew Ward
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 
// Matthew Ward (mrward@users.sourceforge.net)
// 



using System;
using System.Collections;

namespace SoapTrace.Core.Communication
{
	/// <summary>
	///     <para>
	///       A collection that stores <see cref='NetworkInterfaceName'/> objects.
	///    </para>
	/// </summary>
	/// <seealso cref='NetworkInterfaceNameCollection'/>
	[Serializable()]
	public class NetworkInterfaceNameCollection : CollectionBase {
		
		/// <summary>
		///     <para>
		///       Initializes a new instance of <see cref='NetworkInterfaceNameCollection'/>.
		///    </para>
		/// </summary>
		public NetworkInterfaceNameCollection()
		{
		}
		
		/// <summary>
		///     <para>
		///       Initializes a new instance of <see cref='NetworkInterfaceNameCollection'/> based on another <see cref='NetworkInterfaceNameCollection'/>.
		///    </para>
		/// </summary>
		/// <param name='val'>
		///       A <see cref='NetworkInterfaceNameCollection'/> from which the contents are copied
		/// </param>
		public NetworkInterfaceNameCollection(NetworkInterfaceNameCollection val)
		{
			this.AddRange(val);
		}
		
		/// <summary>
		///     <para>
		///       Initializes a new instance of <see cref='NetworkInterfaceNameCollection'/> containing any array of <see cref='NetworkInterfaceName'/> objects.
		///    </para>
		/// </summary>
		/// <param name='val'>
		///       A array of <see cref='NetworkInterfaceName'/> objects with which to intialize the collection
		/// </param>
		public NetworkInterfaceNameCollection(NetworkInterfaceName[] val)
		{
			this.AddRange(val);
		}
		
		/// <summary>
		/// <para>Represents the entry at the specified index of the <see cref='NetworkInterfaceName'/>.</para>
		/// </summary>
		/// <param name='index'><para>The zero-based index of the entry to locate in the collection.</para></param>
		/// <value>
		///    <para> The entry at the specified index of the collection.</para>
		/// </value>
		/// <exception cref='System.ArgumentOutOfRangeException'><paramref name='index'/> is outside the valid range of indexes for the collection.</exception>
		public NetworkInterfaceName this[int index] {
			get {
				return ((NetworkInterfaceName)(List[index]));
			}
			set {
				List[index] = value;
			}
		}
		
		/// <summary>
		///    <para>Adds a <see cref='NetworkInterfaceName'/> with the specified value to the 
		///    <see cref='NetworkInterfaceNameCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='NetworkInterfaceName'/> to add.</param>
		/// <returns>
		///    <para>The index at which the new element was inserted.</para>
		/// </returns>
		/// <seealso cref='NetworkInterfaceNameCollection.AddRange'/>
		public int Add(NetworkInterfaceName val)
		{
			return List.Add(val);
		}
		
		/// <summary>
		/// <para>Copies the elements of an array to the end of the <see cref='NetworkInterfaceNameCollection'/>.</para>
		/// </summary>
		/// <param name='val'>
		///    An array of type <see cref='NetworkInterfaceName'/> containing the objects to add to the collection.
		/// </param>
		/// <returns>
		///   <para>None.</para>
		/// </returns>
		/// <seealso cref='NetworkInterfaceNameCollection.Add'/>
		public void AddRange(NetworkInterfaceName[] val)
		{
			for (int i = 0; i < val.Length; i++) {
				this.Add(val[i]);
			}
		}
		
		/// <summary>
		///     <para>
		///       Adds the contents of another <see cref='NetworkInterfaceNameCollection'/> to the end of the collection.
		///    </para>
		/// </summary>
		/// <param name='val'>
		///    A <see cref='NetworkInterfaceNameCollection'/> containing the objects to add to the collection.
		/// </param>
		/// <returns>
		///   <para>None.</para>
		/// </returns>
		/// <seealso cref='NetworkInterfaceNameCollection.Add'/>
		public void AddRange(NetworkInterfaceNameCollection val)
		{
			for (int i = 0; i < val.Count; i++)
			{
				this.Add(val[i]);
			}
		}
		
		/// <summary>
		/// <para>Gets a value indicating whether the 
		///    <see cref='NetworkInterfaceNameCollection'/> contains the specified <see cref='NetworkInterfaceName'/>.</para>
		/// </summary>
		/// <param name='val'>The <see cref='NetworkInterfaceName'/> to locate.</param>
		/// <returns>
		/// <para><see langword='true'/> if the <see cref='NetworkInterfaceName'/> is contained in the collection; 
		///   otherwise, <see langword='false'/>.</para>
		/// </returns>
		/// <seealso cref='NetworkInterfaceNameCollection.IndexOf'/>
		public bool Contains(NetworkInterfaceName val)
		{
			return List.Contains(val);
		}
		
		/// <summary>
		/// <para>Copies the <see cref='NetworkInterfaceNameCollection'/> values to a one-dimensional <see cref='System.Array'/> instance at the 
		///    specified index.</para>
		/// </summary>
		/// <param name='array'><para>The one-dimensional <see cref='System.Array'/> that is the destination of the values copied from <see cref='NetworkInterfaceNameCollection'/> .</para></param>
		/// <param name='index'>The index in <paramref name='array'/> where copying begins.</param>
		/// <returns>
		///   <para>None.</para>
		/// </returns>
		/// <exception cref='System.ArgumentException'><para><paramref name='array'/> is multidimensional.</para> <para>-or-</para> <para>The number of elements in the <see cref='NetworkInterfaceNameCollection'/> is greater than the available space between <paramref name='arrayIndex'/> and the end of <paramref name='array'/>.</para></exception>
		/// <exception cref='System.ArgumentNullException'><paramref name='array'/> is <see langword='null'/>. </exception>
		/// <exception cref='System.ArgumentOutOfRangeException'><paramref name='arrayIndex'/> is less than <paramref name='array'/>'s lowbound. </exception>
		/// <seealso cref='System.Array'/>
		public void CopyTo(NetworkInterfaceName[] array, int index)
		{
			List.CopyTo(array, index);
		}
		
		/// <summary>
		///    <para>Returns the index of a <see cref='NetworkInterfaceName'/> in 
		///       the <see cref='NetworkInterfaceNameCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='NetworkInterfaceName'/> to locate.</param>
		/// <returns>
		/// <para>The index of the <see cref='NetworkInterfaceName'/> of <paramref name='val'/> in the 
		/// <see cref='NetworkInterfaceNameCollection'/>, if found; otherwise, -1.</para>
		/// </returns>
		/// <seealso cref='NetworkInterfaceNameCollection.Contains'/>
		public int IndexOf(NetworkInterfaceName val)
		{
			return List.IndexOf(val);
		}
		
		/// <summary>
		/// <para>Inserts a <see cref='NetworkInterfaceName'/> into the <see cref='NetworkInterfaceNameCollection'/> at the specified index.</para>
		/// </summary>
		/// <param name='index'>The zero-based index where <paramref name='val'/> should be inserted.</param>
		/// <param name='val'>The <see cref='NetworkInterfaceName'/> to insert.</param>
		/// <returns><para>None.</para></returns>
		/// <seealso cref='NetworkInterfaceNameCollection.Add'/>
		public void Insert(int index, NetworkInterfaceName val)
		{
			List.Insert(index, val);
		}
		
		/// <summary>
		///    <para>Returns an enumerator that can iterate through 
		///       the <see cref='NetworkInterfaceNameCollection'/> .</para>
		/// </summary>
		/// <returns><para>None.</para></returns>
		/// <seealso cref='System.Collections.IEnumerator'/>
		public new NetworkInterfaceNameEnumerator GetEnumerator()
		{
			return new NetworkInterfaceNameEnumerator(this);
		}
		
		/// <summary>
		///    <para> Removes a specific <see cref='NetworkInterfaceName'/> from the 
		///    <see cref='NetworkInterfaceNameCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='NetworkInterfaceName'/> to remove from the <see cref='NetworkInterfaceNameCollection'/> .</param>
		/// <returns><para>None.</para></returns>
		/// <exception cref='System.ArgumentException'><paramref name='val'/> is not found in the Collection. </exception>
		public void Remove(NetworkInterfaceName val)
		{
			List.Remove(val);
		}
		
		/// <summary>
		/// Enumerator for the <see cref="NetworkInterfaceNameCollection"/> class.
		/// </summary>
		public class NetworkInterfaceNameEnumerator : IEnumerator
		{
			IEnumerator baseEnumerator;
			IEnumerable temp;
			
			/// <summary>
			/// Creates a new instance of the <see cref="NetworkInterfaceNameEnumerator"/>
			/// class.
			/// </summary>
			/// <param name="mappings">The collection class that this enumerator will use.</param>
			public NetworkInterfaceNameEnumerator(NetworkInterfaceNameCollection mappings)
			{
				this.temp = ((IEnumerable)(mappings));
				this.baseEnumerator = temp.GetEnumerator();
			}
			
			/// <summary>
			/// Gets the current element in the collection.
			/// </summary>
			public NetworkInterfaceName Current {
				get {
					return ((NetworkInterfaceName)(baseEnumerator.Current));
				}
			}
			
			object IEnumerator.Current {
				get {
					return baseEnumerator.Current;
				}
			}
			
			/// <summary>
			/// Advances the enumerator to the next element of the collection.
			/// </summary>
			/// <returns><see langword='true'/> if the enumerator was successfully advanced to the 
			/// next element; <see langword='false'/> if the enumerator has passed the end of the 
			/// collection.</returns>
			public bool MoveNext()
			{
				return baseEnumerator.MoveNext();
			}
			
			bool IEnumerator.MoveNext()
			{
				return baseEnumerator.MoveNext();
			}
			
			/// <summary>
			/// Sets the enumerator to its initial position, which is before the 
			/// first element in the collection.
			/// </summary>
			public void Reset()
			{
				baseEnumerator.Reset();
			}
			
			void IEnumerator.Reset()
			{
				baseEnumerator.Reset();
			}
		}
	}
}
