// 
// SoapTrace.Net - A soap message monitoring utility. 
// 
// Copyright (C) 2004 Matthew Ward
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 
// Matthew Ward (mrward@users.sourceforge.net)
// 

using System;
using System.Collections;

namespace SoapTrace.Core.Communication
{
	/// <summary>
	/// A type safe collection of <see cref="SoapRequestAndResponse"/> 
	/// objects.
	/// </summary>
	public class SoapRequestAndResponseCollection : CollectionBase
	{
		/// <summary>
		/// Creates a new instance of this class.
		/// </summary>
		public SoapRequestAndResponseCollection( )
		{
		}

		/// <summary>
		/// Gets or sets the item at the specified index.
		/// </summary>
		public SoapRequestAndResponse this[ int index ]
		{
			get
			{
				return ( SoapRequestAndResponse )InnerList[ index ];
			}

			set
			{
				InnerList[ index ] = value;
			}
		}

		/// <summary>
		/// Adds an item to the collection.
		/// </summary>
		/// <param name="item">The item to add.</param>
		public void Add( SoapRequestAndResponse item )
		{
			InnerList.Add( item );
		}

		/// <summary>
		/// Removes an item from the collection.
		/// </summary>
		/// <param name="item">The item to remove.</param>
		public void Remove( SoapRequestAndResponse item )
		{
			InnerList.Remove( item );
		}

		/// <summary>
		/// Gets an object that can be used to synchronise
		/// access to this collection.
		/// </summary>
		public object SyncRoot
		{
			get
			{
				return InnerList.SyncRoot;
			}
		}
		
		/// <summary>
		/// Adds a soap response to the matching request if it does
		/// not contain a response.
		/// </summary>
		/// <param name="clientId">The client ID of the response.</param>
		/// <param name="response">The soap response to add.</param>
		/// <returns>The matching <see cref="SoapRequestAndResponse"/> if
		/// it does not already hold a <see cref="SoapResponse"/> object;
		/// otherwise <see langword="null"/>.</returns>
		public SoapRequestAndResponse AddResponse( ClientId clientId, SoapResponse response )
		{
			SoapRequestAndResponse MatchedItem = null;
			
			for( int i = 0; i < InnerList.Count; ++i )
			{
				SoapRequestAndResponse CurrentItem = this[ i ];

				if( CurrentItem.ClientId == clientId )
				{
					if( CurrentItem.Response == null )
					{
						CurrentItem.Response = response;
						MatchedItem = CurrentItem;
						break;
					}				
				}
			}
			
			return MatchedItem;
		}
		
		/// <summary>
		///   Copies the <see cref='SoapRequestAndResponseCollection'/> values to a one-dimensional <see cref='Array'/> instance at the 
		///    specified index.
		/// </summary>
		/// <param name='array'>The one-dimensional <see cref='Array'/> that is the destination of the values copied from <see cref='SoapRequestAndResponseCollection'/>.</param>
		/// <param name='index'>The index in <paramref name='array'/> where copying begins.</param>
		/// <exception cref='ArgumentException'>
		///   <para><paramref name='array'/> is multidimensional.</para>
		///   <para>-or-</para>
		///   <para>The number of elements in the <see cref='SoapRequestAndResponseCollection'/> is greater than
		///         the available space between <paramref name='arrayIndex'/> and the end of
		///         <paramref name='array'/>.</para>
		/// </exception>
		/// <exception cref='ArgumentNullException'><paramref name='array'/> is <see langword='null'/>. </exception>
		/// <exception cref='ArgumentOutOfRangeException'><paramref name='arrayIndex'/> is less than <paramref name='array'/>'s lowbound. </exception>
		/// <seealso cref='Array'/>
		public void CopyTo(SoapRequestAndResponse[] array, int index)
		{
			List.CopyTo(array, index);
		}	
		
		/// <summary>
		/// <para>Inserts a <see cref='SoapRequestAndResponse'/> into the <see cref='SoapRequestAndResponseCollection'/> at the specified index.</para>
		/// </summary>
		/// <param name='index'>The zero-based index where <paramref name='val'/> should be inserted.</param>
		/// <param name='val'>The <see cref='SoapRequestAndResponse'/> to insert.</param>
		/// <returns><para>None.</para></returns>
		/// <seealso cref='SoapRequestAndResponseCollection.Add'/>
		public void Insert(int index, SoapRequestAndResponse val)
		{
			List.Insert(index, val);
		}
		
		/// <summary>
		///    <para>Returns the index of a <see cref='SoapRequestAndResponse'/> in 
		///       the <see cref='SoapRequestAndResponseCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='SoapRequestAndResponse'/> to locate.</param>
		/// <returns>
		/// <para>The index of the <see cref='SoapRequestAndResponse'/> of <paramref name='val'/> in the 
		/// <see cref='SoapRequestAndResponseCollection'/>, if found; otherwise, -1.</para>
		/// </returns>
		public int IndexOf(SoapRequestAndResponse val)
		{
			return List.IndexOf(val);
		}	
		
		/// <summary>
		/// <para>Gets a value indicating whether the 
		///    <see cref='SoapRequestAndResponseCollection'/> contains the specified <see cref='SoapRequestAndResponse'/>.</para>
		/// </summary>
		/// <param name='val'>The <see cref='SoapRequestAndResponse'/> to locate.</param>
		/// <returns>
		/// <para><see langword='true'/> if the <see cref='SoapRequestAndResponse'/> is contained in the collection; 
		///   otherwise, <see langword='false'/>.</para>
		/// </returns>
		/// <seealso cref='SoapRequestAndResponseCollection.IndexOf'/>
		public bool Contains(SoapRequestAndResponse val)
		{
			return List.Contains(val);
		}
	}
}
