﻿// 
// SoapTrace.Net - A soap message monitoring utility. 
// 
// Copyright (C) 2004 Matthew Ward
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 
// Matthew Ward (mrward@users.sourceforge.net)
// 


using System;
using System.Collections;
using System.Diagnostics;
using System.Net.Sockets;

namespace SoapTrace.Core.Communication
{
	/// <summary>
	///     <para>
	///       A collection that stores <see cref='Socket'/> objects.
	///    </para>
	/// </summary>
	/// <seealso cref='SocketCollection'/>
	public sealed class SocketCollection : CollectionBase 
	{
		
		/// <summary>
		///     <para>
		///       Initializes a new instance of <see cref='SocketCollection'/>.
		///    </para>
		/// </summary>
		public SocketCollection( )
		{
		}
		
		/// <summary>
		///     <para>
		///       Initializes a new instance of <see cref='SocketCollection'/> based on another <see cref='SocketCollection'/>.
		///    </para>
		/// </summary>
		/// <param name='val'>
		///       A <see cref='SocketCollection'/> from which the contents are copied
		/// </param>
		public SocketCollection(SocketCollection val)
		{
			this.AddRange(val);
		}
		
		/// <summary>
		///     <para>
		///       Initializes a new instance of <see cref='SocketCollection'/> containing any array of <see cref='Socket'/> objects.
		///    </para>
		/// </summary>
		/// <param name='val'>
		///       A array of <see cref='Socket'/> objects with which to intialize the collection
		/// </param>
		public SocketCollection(Socket[] val)
		{
			this.AddRange(val);
		}
		
		/// <summary>
		/// <para>Represents the entry at the specified index of the <see cref='Socket'/>.</para>
		/// </summary>
		/// <param name='index'><para>The zero-based index of the entry to locate in the collection.</para></param>
		/// <value>
		///    <para> The entry at the specified index of the collection.</para>
		/// </value>
		/// <exception cref='System.ArgumentOutOfRangeException'><paramref name='index'/> is outside the valid range of indexes for the collection.</exception>
		public Socket this[int index] 
		{
			get 
			{
				return ((Socket)(List[index]));
			}
			set 
			{
				List[index] = value;
			}
		}

		/// <summary>
		/// Gets an object that can be used to synchronize access to the 
		/// <see cref='SocketCollection'/>.
		/// </summary>
		public object SyncRoot
		{
			[ DebuggerStepThrough ]
			get
			{
				return InnerList.SyncRoot;
			}
		}

		/// <summary>
		///    <para>Adds a <see cref='Socket'/> with the specified value to the 
		///    <see cref='SocketCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='Socket'/> to add.</param>
		/// <returns>
		///    <para>The index at which the new element was inserted.</para>
		/// </returns>
		/// <seealso cref='SocketCollection.AddRange'/>
		public int Add(Socket val)
		{
			return List.Add(val);
		}
		
		/// <summary>
		/// <para>Copies the elements of an array to the end of the <see cref='SocketCollection'/>.</para>
		/// </summary>
		/// <param name='val'>
		///    An array of type <see cref='Socket'/> containing the objects to add to the collection.
		/// </param>
		/// <returns>
		///   <para>None.</para>
		/// </returns>
		/// <seealso cref='SocketCollection.Add'/>
		public void AddRange(Socket[] val)
		{
			for (int i = 0; i < val.Length; i++) {
				this.Add(val[i]);
			}
		}
		
		/// <summary>
		///     <para>
		///       Adds the contents of another <see cref='SocketCollection'/> to the end of the collection.
		///    </para>
		/// </summary>
		/// <param name='val'>
		///    A <see cref='SocketCollection'/> containing the objects to add to the collection.
		/// </param>
		/// <returns>
		///   <para>None.</para>
		/// </returns>
		/// <seealso cref='SocketCollection.Add'/>
		public void AddRange(SocketCollection val)
		{
			for (int i = 0; i < val.Count; i++)
			{
				this.Add(val[i]);
			}
		}
		
		/// <summary>
		/// <para>Gets a value indicating whether the 
		///    <see cref='SocketCollection'/> contains the specified <see cref='Socket'/>.</para>
		/// </summary>
		/// <param name='val'>The <see cref='Socket'/> to locate.</param>
		/// <returns>
		/// <para><see langword='true'/> if the <see cref='Socket'/> is contained in the collection; 
		///   otherwise, <see langword='false'/>.</para>
		/// </returns>
		/// <seealso cref='SocketCollection.IndexOf'/>
		public bool Contains(Socket val)
		{
			return List.Contains(val);
		}
		
		/// <summary>
		/// <para>Copies the <see cref='SocketCollection'/> values to a one-dimensional <see cref='System.Array'/> instance at the 
		///    specified index.</para>
		/// </summary>
		/// <param name='array'><para>The one-dimensional <see cref='System.Array'/> that is the destination of the values copied from <see cref='SocketCollection'/> .</para></param>
		/// <param name='index'>The index in <paramref name='array'/> where copying begins.</param>
		/// <returns>
		///   <para>None.</para>
		/// </returns>
		/// <exception cref='System.ArgumentException'><para><paramref name='array'/> is multidimensional.</para> <para>-or-</para> <para>The number of elements in the <see cref='SocketCollection'/> is greater than the available space between <paramref name='arrayIndex'/> and the end of <paramref name='array'/>.</para></exception>
		/// <exception cref='System.ArgumentNullException'><paramref name='array'/> is <see langword='null'/>. </exception>
		/// <exception cref='System.ArgumentOutOfRangeException'><paramref name='arrayIndex'/> is less than <paramref name='array'/>'s lowbound. </exception>
		/// <seealso cref='System.Array'/>
		public void CopyTo(Socket[] array, int index)
		{
			List.CopyTo(array, index);
		}
		
		/// <summary>
		///    <para>Returns the index of a <see cref='Socket'/> in 
		///       the <see cref='SocketCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='Socket'/> to locate.</param>
		/// <returns>
		/// <para>The index of the <see cref='Socket'/> of <paramref name='value'/> in the 
		/// <see cref='SocketCollection'/>, if found; otherwise, -1.</para>
		/// </returns>
		/// <seealso cref='SocketCollection.Contains'/>
		public int IndexOf(Socket val)
		{
			return List.IndexOf(val);
		}
		
		/// <summary>
		/// <para>Inserts a <see cref='Socket'/> into the <see cref='SocketCollection'/> at the specified index.</para>
		/// </summary>
		/// <param name='index'>The zero-based index where <paramref name='value'/> should be inserted.</param>
		/// <param name=' val'>The <see cref='Socket'/> to insert.</param>
		/// <returns><para>None.</para></returns>
		/// <seealso cref='SocketCollection.Add'/>
		public void Insert(int index, Socket val)
		{
			List.Insert(index, val);
		}
		
		/// <summary>
		///    <para>Returns an enumerator that can iterate through 
		///       the <see cref='SocketCollection'/> .</para>
		/// </summary>
		/// <returns><para>None.</para></returns>
		/// <seealso cref='System.Collections.IEnumerator'/>
		public new SocketEnumerator GetEnumerator()
		{
			return new SocketEnumerator(this);
		}
		
		/// <summary>
		///    <para> Removes a specific <see cref='Socket'/> from the 
		///    <see cref='SocketCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='Socket'/> to remove from the <see cref='SocketCollection'/> .</param>
		/// <returns><para>None.</para></returns>
		/// <exception cref='System.ArgumentException'><paramref name='value'/> is not found in the Collection. </exception>
		public void Remove(Socket val)
		{
			List.Remove(val);
		}
		
		/// <summary>
		/// Socket enumerator class.
		/// </summary>
		public class SocketEnumerator : IEnumerator
		{
			IEnumerator baseEnumerator;
			IEnumerable temp;
			
			/// <summary>
			/// Creates a new instance of this class.
			/// </summary>
			/// <param name="mappings">A collection of sockets.</param>
			public SocketEnumerator(SocketCollection mappings)
			{
				this.temp = ((IEnumerable)(mappings));
				this.baseEnumerator = temp.GetEnumerator();
			}
			
			/// <summary>
			/// Gets the item at the current enumerator position.
			/// </summary>
			public Socket Current {
				get {
					return ((Socket)(baseEnumerator.Current));
				}
			}
			
			/// <summary>
			/// Gets the item at the current enumerator position.
			/// </summary>
			object IEnumerator.Current {
				get {
					return baseEnumerator.Current;
				}
			}
			
			/// <summary>
			/// Moves to the next item in the collection.
			/// </summary>
			/// <returns>A status flag indicating whether
			/// the move was successful.</returns>
			public bool MoveNext()
			{
				return baseEnumerator.MoveNext();
			}
			
			/// <summary>
			/// Moves to the next item in the collection.
			/// </summary>
			/// <returns>A status flag indicating whether
			/// the move was successful.</returns>
			bool IEnumerator.MoveNext()
			{
				return baseEnumerator.MoveNext();
			}
			
			/// <summary>
			/// Resets the enumerator.
			/// </summary>
			public void Reset()
			{
				baseEnumerator.Reset();
			}
			
			/// <summary>
			/// Resets the enumerator.
			/// </summary>
			void IEnumerator.Reset()
			{
				baseEnumerator.Reset();
			}
		}
	}
}
