//
// SoapTrace.Net - A soap message monitoring utility. 
//
// Copyright (C) 2005 Matthew Ward
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Matthew Ward (mrward@users.sourceforge.net)


using System;
using System.Collections;
using WinPcap;

namespace SoapTrace.Core.Communication
{
	/// <summary>
	///   A collection that stores <see cref='TcpConnection'/> objects.
	/// </summary>
	[Serializable()]
	public class TcpConnectionCollection : CollectionBase {
		
		/// <summary>
		///   Initializes a new instance of <see cref='TcpConnectionCollection'/>.
		/// </summary>
		public TcpConnectionCollection()
		{
		}
		
		/// <summary>
		///   Initializes a new instance of <see cref='TcpConnectionCollection'/> based on another <see cref='TcpConnectionCollection'/>.
		/// </summary>
		/// <param name='val'>
		///   A <see cref='TcpConnectionCollection'/> from which the contents are copied
		/// </param>
		public TcpConnectionCollection(TcpConnectionCollection val)
		{
			this.AddRange(val);
		}
		
		/// <summary>
		///   Initializes a new instance of <see cref='TcpConnectionCollection'/> containing any array of <see cref='TcpConnection'/> objects.
		/// </summary>
		/// <param name='val'>
		///       A array of <see cref='TcpConnection'/> objects with which to intialize the collection
		/// </param>
		public TcpConnectionCollection(TcpConnection[] val)
		{
			this.AddRange(val);
		}
		
		/// <summary>
		///   Represents the entry at the specified index of the <see cref='TcpConnection'/>.
		/// </summary>
		/// <param name='index'>The zero-based index of the entry to locate in the collection.</param>
		/// <value>The entry at the specified index of the collection.</value>
		/// <exception cref='ArgumentOutOfRangeException'><paramref name='index'/> is outside the valid range of indexes for the collection.</exception>
		public TcpConnection this[int index] {
			get {
				return ((TcpConnection)(List[index]));
			}
			set {
				List[index] = value;
			}
		}
		
		/// <summary>
		/// Gets an object that can be used to synchronise
		/// access to this collection.
		/// </summary>		
		public object SyncRoot
		{
			get
			{
				ICollection Collection = this;
				return Collection.SyncRoot;
			}
		}
		
		/// <summary>
		/// Gets the connection that corresponds to a packet.
		/// </summary>
		/// <param name="packet">A tcp packet.</param>
		/// <returns>The connection associated with the packet; 
		/// otherwise <see langword="null"/>.</returns>
		public TcpConnection FindConnection( TcpPacket packet )
		{
			TcpConnection MatchedConnection = null;
			
			foreach( TcpConnection Connection in this )
			{
				if( Connection.IsMatchingPacket( packet ) )
				{
					MatchedConnection = Connection;
					break;
				}
			}	
			
			return MatchedConnection;
		}
		
		/// <summary>
		///   Adds a <see cref='TcpConnection'/> with the specified value to the 
		///   <see cref='TcpConnectionCollection'/>.
		/// </summary>
		/// <param name='val'>The <see cref='TcpConnection'/> to add.</param>
		/// <returns>The index at which the new element was inserted.</returns>
		/// <seealso cref='TcpConnectionCollection.AddRange'/>
		public int Add(TcpConnection val)
		{
			return List.Add(val);
		}
		
		/// <summary>
		///   Copies the elements of an array to the end of the <see cref='TcpConnectionCollection'/>.
		/// </summary>
		/// <param name='val'>
		///    An array of type <see cref='TcpConnection'/> containing the objects to add to the collection.
		/// </param>
		/// <seealso cref='TcpConnectionCollection.Add'/>
		public void AddRange(TcpConnection[] val)
		{
			for (int i = 0; i < val.Length; i++) {
				this.Add(val[i]);
			}
		}
		
		/// <summary>
		///   Adds the contents of another <see cref='TcpConnectionCollection'/> to the end of the collection.
		/// </summary>
		/// <param name='val'>
		///    A <see cref='TcpConnectionCollection'/> containing the objects to add to the collection.
		/// </param>
		/// <seealso cref='TcpConnectionCollection.Add'/>
		public void AddRange(TcpConnectionCollection val)
		{
			for (int i = 0; i < val.Count; i++)
			{
				this.Add(val[i]);
			}
		}
		
		/// <summary>
		///   Gets a value indicating whether the 
		///    <see cref='TcpConnectionCollection'/> contains the specified <see cref='TcpConnection'/>.
		/// </summary>
		/// <param name='val'>The <see cref='TcpConnection'/> to locate.</param>
		/// <returns>
		/// <see langword='true'/> if the <see cref='TcpConnection'/> is contained in the collection; 
		///   otherwise, <see langword='false'/>.
		/// </returns>
		/// <seealso cref='TcpConnectionCollection.IndexOf'/>
		public bool Contains(TcpConnection val)
		{
			return List.Contains(val);
		}
		
		/// <summary>
		///   Copies the <see cref='TcpConnectionCollection'/> values to a one-dimensional <see cref='Array'/> instance at the 
		///    specified index.
		/// </summary>
		/// <param name='array'>The one-dimensional <see cref='Array'/> that is the destination of the values copied from <see cref='TcpConnectionCollection'/>.</param>
		/// <param name='index'>The index in <paramref name='array'/> where copying begins.</param>
		/// <exception cref='ArgumentException'>
		///   <para><paramref name='array'/> is multidimensional.</para>
		///   <para>-or-</para>
		///   <para>The number of elements in the <see cref='TcpConnectionCollection'/> is greater than
		///         the available space between <paramref name='arrayIndex'/> and the end of
		///         <paramref name='array'/>.</para>
		/// </exception>
		/// <exception cref='ArgumentNullException'><paramref name='array'/> is <see langword='null'/>. </exception>
		/// <exception cref='ArgumentOutOfRangeException'><paramref name='arrayIndex'/> is less than <paramref name='array'/>'s lowbound. </exception>
		/// <seealso cref='Array'/>
		public void CopyTo(TcpConnection[] array, int index)
		{
			List.CopyTo(array, index);
		}
		
		/// <summary>
		///    Returns the index of a <see cref='TcpConnection'/> in 
		///       the <see cref='TcpConnectionCollection'/>.
		/// </summary>
		/// <param name='val'>The <see cref='TcpConnection'/> to locate.</param>
		/// <returns>
		///   The index of the <see cref='TcpConnection'/> of <paramref name='val'/> in the 
		///   <see cref='TcpConnectionCollection'/>, if found; otherwise, -1.
		/// </returns>
		/// <seealso cref='TcpConnectionCollection.Contains'/>
		public int IndexOf(TcpConnection val)
		{
			return List.IndexOf(val);
		}
		
		/// <summary>
		///   Inserts a <see cref='TcpConnection'/> into the <see cref='TcpConnectionCollection'/> at the specified index.
		/// </summary>
		/// <param name='index'>The zero-based index where <paramref name='val'/> should be inserted.</param>
		/// <param name='val'>The <see cref='TcpConnection'/> to insert.</param>
		/// <seealso cref='TcpConnectionCollection.Add'/>
		public void Insert(int index, TcpConnection val)
		{
			List.Insert(index, val);
		}
		
		/// <summary>
		///  Returns an enumerator that can iterate through the <see cref='TcpConnectionCollection'/>.
		/// </summary>
		/// <seealso cref='IEnumerator'/>
		public new TcpConnectionEnumerator GetEnumerator()
		{
			return new TcpConnectionEnumerator(this);
		}
		
		/// <summary>
		///   Removes a specific <see cref='TcpConnection'/> from the <see cref='TcpConnectionCollection'/>.
		/// </summary>
		/// <param name='val'>The <see cref='TcpConnection'/> to remove from the <see cref='TcpConnectionCollection'/>.</param>
		/// <exception cref='ArgumentException'><paramref name='val'/> is not found in the Collection.</exception>
		public void Remove(TcpConnection val)
		{
			List.Remove(val);
		}
		
		/// <summary>
		///   Enumerator that can iterate through a TcpConnectionCollection.
		/// </summary>
		/// <seealso cref='IEnumerator'/>
		/// <seealso cref='TcpConnectionCollection'/>
		/// <seealso cref='TcpConnection'/>
		public class TcpConnectionEnumerator : IEnumerator
		{
			IEnumerator baseEnumerator;
			IEnumerable temp;
			
			/// <summary>
			///   Initializes a new instance of <see cref='TcpConnectionEnumerator'/>.
			/// </summary>
			public TcpConnectionEnumerator(TcpConnectionCollection mappings)
			{
				this.temp = ((IEnumerable)(mappings));
				this.baseEnumerator = temp.GetEnumerator();
			}
			
			/// <summary>
			///   Gets the current <see cref='TcpConnection'/> in the <seealso cref='TcpConnectionCollection'/>.
			/// </summary>
			public TcpConnection Current {
				get {
					return ((TcpConnection)(baseEnumerator.Current));
				}
			}
			
			object IEnumerator.Current {
				get {
					return baseEnumerator.Current;
				}
			}
			
			/// <summary>
			///   Advances the enumerator to the next <see cref='TcpConnection'/> of the <see cref='TcpConnectionCollection'/>.
			/// </summary>
			public bool MoveNext()
			{
				return baseEnumerator.MoveNext();
			}
			
			/// <summary>
			///   Sets the enumerator to its initial position, which is before the first element in the <see cref='TcpConnectionCollection'/>.
			/// </summary>
			public void Reset()
			{
				baseEnumerator.Reset();
			}
		}
	}
}
