//
// SoapTrace.Net - A soap message monitoring utility. 
//
// Copyright (C) 2005 Matthew Ward
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Matthew Ward (mrward@users.sourceforge.net)


using System;
using System.Collections;

namespace SoapTrace.Core.Diagnostics
{
	/// <summary>
	///   A collection that stores <see cref='PacketLogEntry'/> objects.
	/// </summary>
	[Serializable()]
	public class PacketLogEntryCollection : CollectionBase {
		
		/// <summary>
		/// The default count limit.
		/// </summary>
		public const int DefaultCountLimit = 50;
		
		private int m_CountLimit = DefaultCountLimit;
		
		/// <summary>
		///   Initializes a new instance of <see cref='PacketLogEntryCollection'/>.
		/// </summary>
		public PacketLogEntryCollection()
		{
		}
		
		/// <summary>
		///   Initializes a new instance of <see cref='PacketLogEntryCollection'/> based on another <see cref='PacketLogEntryCollection'/>.
		/// </summary>
		/// <param name='val'>
		///   A <see cref='PacketLogEntryCollection'/> from which the contents are copied
		/// </param>
		public PacketLogEntryCollection(PacketLogEntryCollection val)
		{
			this.AddRange(val);
		}
		
		/// <summary>
		///   Initializes a new instance of <see cref='PacketLogEntryCollection'/> containing any array of <see cref='PacketLogEntry'/> objects.
		/// </summary>
		/// <param name='val'>
		///       A array of <see cref='PacketLogEntry'/> objects with which to intialize the collection
		/// </param>
		public PacketLogEntryCollection(PacketLogEntry[] val)
		{
			this.AddRange(val);
		}
		
		/// <summary>
		///   Represents the entry at the specified index of the <see cref='PacketLogEntry'/>.
		/// </summary>
		/// <param name='index'>The zero-based index of the entry to locate in the collection.</param>
		/// <value>The entry at the specified index of the collection.</value>
		/// <exception cref='ArgumentOutOfRangeException'><paramref name='index'/> is outside the valid range of indexes for the collection.</exception>
		public PacketLogEntry this[int index] {
			get {
				return ((PacketLogEntry)(List[index]));
			}
			set {
				List[index] = value;
			}
		}
		
		/// <summary>
		/// Gets an object that can be used to synchronize access to the
		/// <see cref="PacketLogEntryCollection"/>.
		/// </summary>
		public object SyncRoot
		{
			get
			{
				ICollection Collection = this;
				return Collection.SyncRoot;
			}
		}
		
		/// <summary>
		/// Gets or sets the maximum number of items allowed in this collection.
		/// </summary>
		public int CountLimit
		{
			get
			{
				return m_CountLimit;
			}
			
			set
			{
				m_CountLimit = value;
			}
		}
		
		/// <summary>
		///   Adds a <see cref='PacketLogEntry'/> with the specified value to the 
		///   <see cref='PacketLogEntryCollection'/>.
		/// </summary>
		/// <param name='val'>The <see cref='PacketLogEntry'/> to add.</param>
		/// <returns>The index at which the new element was inserted.</returns>
		/// <seealso cref='PacketLogEntryCollection.AddRange'/>
		public int Add(PacketLogEntry val)
		{
			int Index = -1;
			
			if( m_CountLimit > 0 )
			{							
				// Remove extra entries.
				
				while( List.Count >= m_CountLimit )
				{
					List.RemoveAt( 0 );
				}
				
				Index = List.Add(val);
			}
			
			return Index;
		}
		
		/// <summary>
		///   Copies the elements of an array to the end of the <see cref='PacketLogEntryCollection'/>.
		/// </summary>
		/// <param name='val'>
		///    An array of type <see cref='PacketLogEntry'/> containing the objects to add to the collection.
		/// </param>
		/// <seealso cref='PacketLogEntryCollection.Add'/>
		public void AddRange(PacketLogEntry[] val)
		{
			for (int i = 0; i < val.Length; i++) {
				this.Add(val[i]);
			}
		}
		
		/// <summary>
		///   Adds the contents of another <see cref='PacketLogEntryCollection'/> to the end of the collection.
		/// </summary>
		/// <param name='val'>
		///    A <see cref='PacketLogEntryCollection'/> containing the objects to add to the collection.
		/// </param>
		/// <seealso cref='PacketLogEntryCollection.Add'/>
		public void AddRange(PacketLogEntryCollection val)
		{
			for (int i = 0; i < val.Count; i++)
			{
				this.Add(val[i]);
			}
		}
		
		/// <summary>
		///   Gets a value indicating whether the 
		///    <see cref='PacketLogEntryCollection'/> contains the specified <see cref='PacketLogEntry'/>.
		/// </summary>
		/// <param name='val'>The <see cref='PacketLogEntry'/> to locate.</param>
		/// <returns>
		/// <see langword='true'/> if the <see cref='PacketLogEntry'/> is contained in the collection; 
		///   otherwise, <see langword='false'/>.
		/// </returns>
		/// <seealso cref='PacketLogEntryCollection.IndexOf'/>
		public bool Contains(PacketLogEntry val)
		{
			return List.Contains(val);
		}
		
		/// <summary>
		///   Copies the <see cref='PacketLogEntryCollection'/> values to a one-dimensional <see cref='Array'/> instance at the 
		///    specified index.
		/// </summary>
		/// <param name='array'>The one-dimensional <see cref='Array'/> that is the destination of the values copied from <see cref='PacketLogEntryCollection'/>.</param>
		/// <param name='index'>The index in <paramref name='array'/> where copying begins.</param>
		/// <exception cref='ArgumentException'>
		///   <para><paramref name='array'/> is multidimensional.</para>
		///   <para>-or-</para>
		///   <para>The number of elements in the <see cref='PacketLogEntryCollection'/> is greater than
		///         the available space between <paramref name='arrayIndex'/> and the end of
		///         <paramref name='array'/>.</para>
		/// </exception>
		/// <exception cref='ArgumentNullException'><paramref name='array'/> is <see langword='null'/>. </exception>
		/// <exception cref='ArgumentOutOfRangeException'><paramref name='arrayIndex'/> is less than <paramref name='array'/>'s lowbound. </exception>
		/// <seealso cref='Array'/>
		public void CopyTo(PacketLogEntry[] array, int index)
		{
			List.CopyTo(array, index);
		}
		
		/// <summary>
		///    Returns the index of a <see cref='PacketLogEntry'/> in 
		///       the <see cref='PacketLogEntryCollection'/>.
		/// </summary>
		/// <param name='val'>The <see cref='PacketLogEntry'/> to locate.</param>
		/// <returns>
		///   The index of the <see cref='PacketLogEntry'/> of <paramref name='val'/> in the 
		///   <see cref='PacketLogEntryCollection'/>, if found; otherwise, -1.
		/// </returns>
		/// <seealso cref='PacketLogEntryCollection.Contains'/>
		public int IndexOf(PacketLogEntry val)
		{
			return List.IndexOf(val);
		}
		
		/// <summary>
		///   Inserts a <see cref='PacketLogEntry'/> into the <see cref='PacketLogEntryCollection'/> at the specified index.
		/// </summary>
		/// <param name='index'>The zero-based index where <paramref name='val'/> should be inserted.</param>
		/// <param name='val'>The <see cref='PacketLogEntry'/> to insert.</param>
		/// <seealso cref='PacketLogEntryCollection.Add'/>
		public void Insert(int index, PacketLogEntry val)
		{
			List.Insert(index, val);
		}
		
		/// <summary>
		///  Returns an enumerator that can iterate through the <see cref='PacketLogEntryCollection'/>.
		/// </summary>
		/// <seealso cref='IEnumerator'/>
		public new PacketLogEntryEnumerator GetEnumerator()
		{
			return new PacketLogEntryEnumerator(this);
		}
		
		/// <summary>
		///   Removes a specific <see cref='PacketLogEntry'/> from the <see cref='PacketLogEntryCollection'/>.
		/// </summary>
		/// <param name='val'>The <see cref='PacketLogEntry'/> to remove from the <see cref='PacketLogEntryCollection'/>.</param>
		/// <exception cref='ArgumentException'><paramref name='val'/> is not found in the Collection.</exception>
		public void Remove(PacketLogEntry val)
		{
			List.Remove(val);
		}
		
		/// <summary>
		///   Enumerator that can iterate through a PacketLogEntryCollection.
		/// </summary>
		/// <seealso cref='IEnumerator'/>
		/// <seealso cref='PacketLogEntryCollection'/>
		/// <seealso cref='PacketLogEntry'/>
		public class PacketLogEntryEnumerator : IEnumerator
		{
			IEnumerator baseEnumerator;
			IEnumerable temp;
			
			/// <summary>
			///   Initializes a new instance of <see cref='PacketLogEntryEnumerator'/>.
			/// </summary>
			public PacketLogEntryEnumerator(PacketLogEntryCollection mappings)
			{
				this.temp = ((IEnumerable)(mappings));
				this.baseEnumerator = temp.GetEnumerator();
			}
			
			/// <summary>
			///   Gets the current <see cref='PacketLogEntry'/> in the <seealso cref='PacketLogEntryCollection'/>.
			/// </summary>
			public PacketLogEntry Current {
				get {
					return ((PacketLogEntry)(baseEnumerator.Current));
				}
			}
			
			object IEnumerator.Current {
				get {
					return baseEnumerator.Current;
				}
			}
			
			/// <summary>
			///   Advances the enumerator to the next <see cref='PacketLogEntry'/> of the <see cref='PacketLogEntryCollection'/>.
			/// </summary>
			public bool MoveNext()
			{
				return baseEnumerator.MoveNext();
			}
			
			/// <summary>
			///   Sets the enumerator to its initial position, which is before the first element in the <see cref='PacketLogEntryCollection'/>.
			/// </summary>
			public void Reset()
			{
				baseEnumerator.Reset();
			}
		}
	}
}
