//
// SoapTrace.Net - A soap message monitoring utility. 
//
// Copyright (C) 2005 Matthew Ward
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Matthew Ward (mrward@users.sourceforge.net)

using System;
using System.Xml.Serialization;

namespace SoapTrace.Core.Diagnostics
{
	/// <summary>
	/// Holds the configuration for the <see cref="SoapMessageLogger"/>.
	/// </summary>
	[ XmlType( "SoapMessageLogger" ) ]
	public class SoapMessageLoggerConfiguration
	{
		/// <summary>
		/// The full name of the type ( 'Type,Assembly' ).
		/// </summary>
		private string m_FullTypeName = String.Empty;

		/// <summary>
		/// The name of the type.
		/// </summary>
		private string m_TypeName = String.Empty;

		/// <summary>
		/// The name of the assembly.
		/// </summary>
		private string m_AssemblyName = String.Empty;
		
		/// <summary>
		/// The name of the log folder.
		/// </summary>
		private string m_Folder = String.Empty;

		/// <summary>
		/// Logging enabled status flag.
		/// </summary>
		bool m_Enabled;
		
		/// <summary>
		/// Creates a new instance of the 
		/// <see cref="SoapMessageLoggerConfiguration"/> class.
		/// </summary>
		public SoapMessageLoggerConfiguration( )
		{
		}
		
		/// <summary>
		/// Gets or sets the enabled status flag.
		/// </summary>
		[ XmlAttribute( "enabled" ) ]
		public bool Enabled
		{
			get
			{
				return m_Enabled;
			}
			
			set
			{
				m_Enabled = value;
			}
		}
		
		/// <summary>
		/// Gets or sets the full name of the type.
		/// </summary>
		[ XmlAttribute( "type" ) ]
		public string FullTypeName
		{
			get
			{
				return m_FullTypeName;
			}

			set
			{
				ParseFullTypeName( value );
				m_FullTypeName = value;
			}
		}
		
		/// <summary>
		/// Gets or sets the logging folder.
		/// </summary>
		public string Folder
		{
			get
			{
				return m_Folder;				
			}
			
			set
			{
				m_Folder = value;
			}
		}

		/// <summary>
		/// Gets or sets the name of the type.
		/// </summary>
		[ XmlIgnore ]
		public string TypeName
		{
			get
			{
				return m_TypeName;
			}

			set
			{
				m_TypeName = value;
			}
		}

		/// <summary>
		/// Gets or sets the assembly of the type.
		/// </summary>
		[ XmlIgnore ]
		public string AssemblyName
		{
			get
			{
				return m_AssemblyName;
			}

			set
			{
				m_AssemblyName = value;
			}
		}

		/// <summary>
		/// Splits up the full type name into type and assembly parts.
		/// </summary>
		/// <param name="fullTypeName">The full type name string.</param>
		private void ParseFullTypeName( string fullTypeName )
		{
			int Separator = fullTypeName.IndexOf( "," );
			if( ( Separator >= 0 ) && ( Separator < ( fullTypeName.Length - 1 ) ) )
			{
				m_TypeName = fullTypeName.Substring( 0, Separator ).Trim( );
				m_AssemblyName = fullTypeName.Substring( ( Separator + 1 ) ).Trim( );
			}
			else
			{
				m_TypeName = String.Empty;
				m_AssemblyName = String.Empty;
			}
		}
	}
}
