// 
// SoapTrace.Net - A soap message monitoring utility. 
// 
// Copyright (C) 2004 Matthew Ward
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 
// Matthew Ward (mrward@users.sourceforge.net)
// 

using System;
using System.Collections;

namespace SoapTrace.Core.Formatters
{
	/// <summary>
	/// Stores a collection of MessageFormatterEntry objects.
	/// </summary>
	public class MessageFormatterEntryCollection : CollectionBase
	{
		/// <summary>
		/// Creates a new instance of the <see cref="MessageFormatterEntryCollection"/>
		/// class.
		/// </summary>
		public MessageFormatterEntryCollection( )
		{
		}

		/// <summary>
		/// Gets the formatter at the specified index.
		/// </summary>
		public MessageFormatterEntry this[ int index ]
		{
			get
			{
				return ( MessageFormatterEntry )InnerList[ index ];
			}
		}

		/// <summary>
		/// Adds an entry to the collection.
		/// </summary>
		/// <param name="entry"></param>
		public void Add( MessageFormatterEntry entry )
		{
			InnerList.Add( entry );
		}
		
		/// <summary>
		///   Copies the <see cref='MessageFormatterEntryCollection'/> values to a one-dimensional <see cref='Array'/> instance at the 
		///    specified index.
		/// </summary>
		/// <param name='array'>The one-dimensional <see cref='Array'/> that is the destination of the values copied from <see cref='MessageFormatterEntryCollection'/>.</param>
		/// <param name='index'>The index in <paramref name='array'/> where copying begins.</param>
		/// <exception cref='ArgumentException'>
		///   <para><paramref name='array'/> is multidimensional.</para>
		///   <para>-or-</para>
		///   <para>The number of elements in the <see cref='MessageFormatterEntryCollection'/> is greater than
		///         the available space between <paramref name='arrayIndex'/> and the end of
		///         <paramref name='array'/>.</para>
		/// </exception>
		/// <exception cref='ArgumentNullException'><paramref name='array'/> is <see langword='null'/>. </exception>
		/// <exception cref='ArgumentOutOfRangeException'><paramref name='arrayIndex'/> is less than <paramref name='array'/>'s lowbound. </exception>
		/// <seealso cref='Array'/>
		public void CopyTo(MessageFormatterEntry[] array, int index)
		{
			List.CopyTo(array, index);
		}
		
		/// <summary>
		///   Removes a specific <see cref='MessageFormatterEntry'/> from the <see cref='MessageFormatterEntryCollection'/>.
		/// </summary>
		/// <param name='val'>The <see cref='MessageFormatterEntry'/> to remove from the <see cref='MessageFormatterEntryCollection'/>.</param>
		/// <exception cref='ArgumentException'><paramref name='val'/> is not found in the Collection.</exception>
		public void Remove(MessageFormatterEntry val)
		{
			List.Remove(val);
		}	
		
		/// <summary>
		/// <para>Inserts a <see cref='MessageFormatterEntry'/> into the <see cref='MessageFormatterEntryCollection'/> at the specified index.</para>
		/// </summary>
		/// <param name='index'>The zero-based index where <paramref name='val'/> should be inserted.</param>
		/// <param name='val'>The <see cref='MessageFormatterEntry'/> to insert.</param>
		/// <returns><para>None.</para></returns>
		/// <seealso cref='MessageFormatterEntryCollection.Add'/>
		public void Insert(int index, MessageFormatterEntry val)
		{
			List.Insert(index, val);
		}		
		
		/// <summary>
		///    <para>Returns the index of a <see cref='MessageFormatterEntry'/> in 
		///       the <see cref='MessageFormatterEntryCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='MessageFormatterEntry'/> to locate.</param>
		/// <returns>
		/// <para>The index of the <see cref='MessageFormatterEntry'/> of <paramref name='val'/> in the 
		/// <see cref='MessageFormatterEntryCollection'/>, if found; otherwise, -1.</para>
		/// </returns>
		public int IndexOf(MessageFormatterEntry val)
		{
			return List.IndexOf(val);
		}	
		
		/// <summary>
		/// <para>Gets a value indicating whether the 
		///    <see cref='MessageFormatterEntryCollection'/> contains the specified <see cref='MessageFormatterEntry'/>.</para>
		/// </summary>
		/// <param name='val'>The <see cref='MessageFormatterEntry'/> to locate.</param>
		/// <returns>
		/// <para><see langword='true'/> if the <see cref='MessageFormatterEntry'/> is contained in the collection; 
		///   otherwise, <see langword='false'/>.</para>
		/// </returns>
		/// <seealso cref='MessageFormatterEntryCollection.IndexOf'/>
		public bool Contains(MessageFormatterEntry val)
		{
			return List.Contains(val);
		}		
	}
}
