// 
// SoapTrace.Net - A soap message monitoring utility. 
// 
// Copyright (C) 2004 Matthew Ward
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 
// Matthew Ward (mrward@users.sourceforge.net)
// 

using System;
using System.Collections;

namespace SoapTrace.Core.Formatters
{
	/// <summary>
	/// A type safe collection of MessageFormatters objects
	/// </summary>
	public class MessageFormattersCollection : CollectionBase
	{
		/// <summary>
		/// Creates a new instance of this class.
		/// </summary>
		public MessageFormattersCollection( )
		{
		}

		/// <summary>
		/// Gets a message formatter at the specified index.
		/// </summary>
		public IMessageFormatter this[ int index ]
		{
			get
			{
				return ( IMessageFormatter )InnerList[ index ];
			}
		}

		/// <summary>
		/// Gets a message formatter based on its name.
		/// </summary>
		public IMessageFormatter this[ string name ]
		{
			get
			{
				IMessageFormatter MatchedFormatter = null;

				foreach( IMessageFormatter Formatter in InnerList )
				{
					if( Formatter.Name == name )
					{
						MatchedFormatter = Formatter;
						break;
					}
				}

				return MatchedFormatter;
			}
		}

		/// <summary>
		/// Adds a message formatter to the collection.
		/// </summary>
		/// <param name="formatter">The message formatter to be added.</param>
		public void Add( IMessageFormatter formatter )
		{
			InnerList.Add( formatter );
		}

		/// <summary>
		/// Determines whether the specified formatter can be configured.
		/// </summary>
		/// <param name="name">The name of the message formatter.</param>
		/// <returns>A flag that indicates whether the message
		/// formatter can be configured.</returns>
		public bool IsConfigurable( string name )
		{
			bool IsConfigurable = false;

			IMessageFormatter Formatter = this[ name ];
			IsConfigurable = Formatter.IsConfigurable;

			return IsConfigurable;
		}
		
		/// <summary>
		///   Copies the <see cref='MessageFormattersCollection'/> values to a one-dimensional <see cref='Array'/> instance at the 
		///    specified index.
		/// </summary>
		/// <param name='array'>The one-dimensional <see cref='Array'/> that is the destination of the values copied from <see cref='MessageFormattersCollection'/>.</param>
		/// <param name='index'>The index in <paramref name='array'/> where copying begins.</param>
		/// <exception cref='ArgumentException'>
		///   <para><paramref name='array'/> is multidimensional.</para>
		///   <para>-or-</para>
		///   <para>The number of elements in the <see cref='MessageFormattersCollection'/> is greater than
		///         the available space between <paramref name='arrayIndex'/> and the end of
		///         <paramref name='array'/>.</para>
		/// </exception>
		/// <exception cref='ArgumentNullException'><paramref name='array'/> is <see langword='null'/>. </exception>
		/// <exception cref='ArgumentOutOfRangeException'><paramref name='arrayIndex'/> is less than <paramref name='array'/>'s lowbound. </exception>
		/// <seealso cref='Array'/>
		public void CopyTo(IMessageFormatter[] array, int index)
		{
			List.CopyTo(array, index);
		}
		
		/// <summary>
		///   Removes a specific <see cref='IMessageFormatter'/> from the <see cref='MessageFormattersCollection'/>.
		/// </summary>
		/// <param name='val'>The <see cref='IMessageFormatter'/> to remove from the <see cref='MessageFormattersCollection'/>.</param>
		/// <exception cref='ArgumentException'><paramref name='val'/> is not found in the Collection.</exception>
		public void Remove(IMessageFormatter val)
		{
			List.Remove(val);
		}	
		
		/// <summary>
		/// <para>Inserts a <see cref='IMessageFormatter'/> into the <see cref='MessageFormattersCollection'/> at the specified index.</para>
		/// </summary>
		/// <param name='index'>The zero-based index where <paramref name='val'/> should be inserted.</param>
		/// <param name='val'>The <see cref='IMessageFormatter'/> to insert.</param>
		/// <returns><para>None.</para></returns>
		/// <seealso cref='MessageFormattersCollection.Add'/>
		public void Insert(int index, IMessageFormatter val)
		{
			List.Insert(index, val);
		}
		
		/// <summary>
		///    <para>Returns the index of a <see cref='WebServiceArrayItem'/> in 
		///       the <see cref='MessageFormattersCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='IMessageFormatter'/> to locate.</param>
		/// <returns>
		/// <para>The index of the <see cref='IMessageFormatter'/> of <paramref name='val'/> in the 
		/// <see cref='MessageFormattersCollection'/>, if found; otherwise, -1.</para>
		/// </returns>
		public int IndexOf(IMessageFormatter val)
		{
			return List.IndexOf(val);
		}	
		
		/// <summary>
		/// <para>Gets a value indicating whether the 
		///    <see cref='MessageFormattersCollection'/> contains the specified <see cref='WebServiceArrayItem'/>.</para>
		/// </summary>
		/// <param name='val'>The <see cref='IMessageFormatter'/> to locate.</param>
		/// <returns>
		/// <para><see langword='true'/> if the <see cref='IMessageFormatter'/> is contained in the collection; 
		///   otherwise, <see langword='false'/>.</para>
		/// </returns>
		/// <seealso cref='MessageFormattersCollection.IndexOf'/>
		public bool Contains(IMessageFormatter val)
		{
			return List.Contains(val);
		}		
	}
}
