//
// SoapTrace.Net - A soap message monitoring utility. 
//
// Copyright (C) 2005 Matthew Ward
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Matthew Ward (mrward@users.sourceforge.net)


using System;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using System.Xml;

namespace SoapTrace.Core.Formatters
{
	/// <summary>
	/// Formats a <see cref="RichTextBox"/> containing xml.
	/// </summary>
	public class RichTextBoxXmlFormatter : RichTextBoxFormatter
	{
		/// <summary>
		/// The markup delimiter colour (uses IE defaults).
		/// </summary>
		private Color m_MarkupDelimiterColour = Color.Blue;
		
		/// <summary>
		/// The element name colour (Uses IE defaults).
		/// </summary>
		private Color m_ElementColour = Color.FromArgb( 153, 0, 0 );
		
		/// <summary>
		/// The attribute value's colour.
		/// </summary>
		private Color m_AttributeValueColour = Color.Black;
		
		/// <summary>
		/// The attribute colour.
		/// </summary>
		private Color m_AttributeColour = Color.FromArgb( 153, 0, 0 );
		
		/// <summary>
		/// The element's text colour.
		/// </summary>
		private Color m_ElementTextColour = Color.Black;
		
		/// <summary>
		/// The comment colour (Uses IE default).
		/// </summary>
		private Color m_CommentColour = Color.FromArgb( 136, 136, 136 );
		
		/// <summary>
		/// The CData text colour.
		/// </summary>
		private Color m_CDataColour = Color.Black;

		/// <summary>
		/// Creates a new instance of the <see cref="RichTextBoxXmlFormatter"/>
		/// class.
		/// </summary>
		public RichTextBoxXmlFormatter( )
		{
		}
		
		/// <summary>
		/// Gets or sets the text colour to use for markup delimiters.
		/// </summary>
		public Color MarkupDelimiterColour
		{
			get
			{
				return m_MarkupDelimiterColour;
			}
			
			set
			{
				m_MarkupDelimiterColour = value;
			}
		}
	
		/// <summary>
		/// Gets or sets the text colour to use for elements. 
		/// </summary>
		public Color ElementColour
		{
			get
			{
				return m_ElementColour;
			}
			
			set
			{
				m_ElementColour = value;
			}
		}	
		
		/// <summary>
		/// Gets or sets the text colour to use for attribute values. 
		/// </summary>
		public Color AttributeValueColour
		{
			get
			{
				return m_AttributeValueColour;
			}
			
			set
			{
				m_AttributeValueColour = value;
			}
		}		
		
		/// <summary>
		/// Gets or sets the text colour to use for an element's text. 
		/// </summary>
		public Color ElementTextColour
		{
			get
			{
				return m_ElementTextColour;
			}
			
			set
			{
				m_ElementTextColour = value;
			}
		}		
		
		/// <summary>
		/// Gets or sets the text colour to use for comment text. 
		/// </summary>
		public Color CommentColour
		{
			get
			{
				return m_CommentColour;
			}
			
			set
			{
				m_CommentColour = value;
			}
		}		
		
		/// <summary>
		/// Gets or sets the text colour to use for CData text. 
		/// </summary>
		public Color CDataColour
		{
			get
			{
				return m_CDataColour;
			}
			
			set
			{
				m_CDataColour = value;
			}
		}			
		
		/// <summary>
		/// Gets or sets the text colour to use for attribute names. 
		/// </summary>
		public Color AttributeColour
		{
			get
			{
				return m_AttributeColour;
			}
			
			set
			{
				m_AttributeColour = value;
			}
		}		
		
		/// <summary>
		/// Formats the <see cref="RichTextBox"/> that contains
		/// xml.
		/// </summary>
		/// <param name="textBox">The <see cref="RichTextBox"/> containing
		/// xml.</param>
		/// <param name="startIndex">The character index that the 
		/// xml starts.</param>
		/// <param name="length">The number of characters to format.</param>
		public void Format( RichTextBox textBox, int startIndex, int length )
		{
			try
			{	
				textBox.Select( startIndex, length );
				textBox.SelectionColor = m_MarkupDelimiterColour;
							
				int BaseLineIndex = textBox.GetLineFromCharIndex( startIndex );
				
				StringReader StringReader = new StringReader( textBox.SelectedText );
				XmlTextReader XmlTextReader = new XmlTextReader( StringReader );
				
				while( XmlTextReader.Read( ) )
				{					
					switch( XmlTextReader.NodeType )
					{
				    	case XmlNodeType.Element:
						{
							SetTextColour( m_ElementColour, textBox, BaseLineIndex + XmlTextReader.LineNumber, XmlTextReader.LinePosition, XmlTextReader.Name.Length );
						
							if( XmlTextReader.HasAttributes )
							{
								SetAttributesTextColour( textBox, BaseLineIndex, XmlTextReader );
							}
						}
						break;
						
						case XmlNodeType.Text:
						{
							SetTextColour( m_ElementTextColour, textBox, BaseLineIndex + XmlTextReader.LineNumber, XmlTextReader.LinePosition, XmlTextReader.Value.Length );							
						}
						break;
						
						case XmlNodeType.Comment:
						{
							SetTextColour( m_CommentColour, textBox, BaseLineIndex + XmlTextReader.LineNumber, XmlTextReader.LinePosition, XmlTextReader.Value.Length );							
						}
						break;
						
						case XmlNodeType.CDATA:
						{
							SetTextColour( m_CDataColour, textBox, BaseLineIndex + XmlTextReader.LineNumber, XmlTextReader.LinePosition, XmlTextReader.Value.Length );							
						}
						break;					
					
						case XmlNodeType.EndElement:
						{
							SetTextColour( m_ElementColour, textBox, BaseLineIndex + XmlTextReader.LineNumber, XmlTextReader.LinePosition, XmlTextReader.Name.Length );
						}
						break;
					}
				}
				
				XmlTextReader.Close( );
			}
			catch( XmlException Ex )
			{
				// Ignore.
				
				System.Diagnostics.Debug.WriteLine( Ex.Message );
			}
			finally
			{
				// Put the cursor at the end of the text.
				textBox.Select( textBox.Text.Length, 0 );
			}
		}
	
		/// <summary>
		/// Sets the attribute value colour for the current XmlTextReader node.
		/// </summary>
		/// <param name="textBox">The rich text box to format.</param>
		/// <param name="baseLineIndex">The line index offset.</param>
		/// <param name="xmlTextReader">The XmlTextReader at the current node 
		/// containing attributes.</param>
		private void SetAttributesTextColour( RichTextBox textBox, int baseLineIndex, XmlTextReader xmlTextReader )
		{
			for( int i = 0; i < xmlTextReader.AttributeCount; ++i )
			{
				xmlTextReader.MoveToAttribute( i );
				
				SetTextColour( m_AttributeColour, textBox, baseLineIndex + xmlTextReader.LineNumber, xmlTextReader.LinePosition, xmlTextReader.Name.Length );
				
				xmlTextReader.ReadAttributeValue( );
				
				SetTextColour( m_AttributeValueColour, textBox, baseLineIndex + xmlTextReader.LineNumber, xmlTextReader.LinePosition, xmlTextReader.Value.Length );
			}
		}

	}
}
