// 
// SoapTrace.Net - A soap message monitoring utility. 
// 
// Copyright (C) 2004 Matthew Ward
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 
// Matthew Ward (mrward@users.sourceforge.net)
// 

using System;
using System.Collections;
using System.Globalization;

namespace SoapTrace.Core.Formatters
{
	/// <summary>
	/// A collection of WebServices objects.
	/// </summary>
	public class WebServiceCollection : CollectionBase, ICloneable
	{
		/// <summary>
		/// Creates a new instance of the <see cref="WebServiceCollection"/>
		/// class.
		/// </summary>
		public WebServiceCollection( )
		{
		}

		/// <summary>
		/// Adds a web service to the collection.
		/// </summary>
		/// <param name="service">The web service to add.</param>
		public void Add( WebService service )
		{
			InnerList.Add( service );
		}

		/// <summary>
		/// Gets or sets the web service at the specified index.
		/// </summary>
		public WebService this[ int index ]
		{
			get
			{
				return ( WebService )InnerList[ index ];
			}

			set
			{
				InnerList[ index ] = value;
			}
		}

		/// <summary>
		/// Finds a web service via its name.
		/// </summary>
		/// <param name="serviceName">The web service name to find.</param>
		/// <returns>Returns the corresponding web service object or null if 
		/// no matches were found.</returns>
		public WebService Find( string serviceName )
		{
			WebService MatchedWebService = null;
	
			for( int i = 0; ( MatchedWebService == null ) && ( i < InnerList.Count ); ++i )
			{
				WebService WebService = ( WebService )InnerList[ i ];
			
				if( String.Compare( WebService.Name, serviceName, true, CultureInfo.InvariantCulture ) == 0 )
				{
					MatchedWebService = WebService;
				}
			}

			return MatchedWebService;
		}

		/// <summary>
		/// Finds a web service via its uri.
		/// </summary>
		/// <param name="uri">The web service uri.</param>
		/// <returns>Returns the corresponding web service object or null if 
		/// no matches were found.</returns>
		public WebService FindByUri( string uri )
		{
			WebService MatchedWebService = null;
	
			for( int i = 0; ( MatchedWebService == null ) && ( i < InnerList.Count ); ++i )
			{
				WebService WebService = ( WebService )InnerList[ i ];
			
				if( String.Compare( WebService.Uri, uri, true, CultureInfo.InvariantCulture ) == 0 )
				{
					MatchedWebService = WebService;
				}
			}

			return MatchedWebService;
		}

		#region ICloneable Members

		/// <summary>
		/// Clones the web service collection.  This does a deep copy
		/// of the web service collection and its contents.
		/// </summary>
		/// <returns></returns>
		public object Clone( )
		{
			WebServiceCollection WebServices = new WebServiceCollection( );

			foreach( WebService WebService in this )
			{
				WebServices.Add( ( WebService )WebService.Clone( ) );
			}

			return WebServices;
		}

		#endregion
	
		/// <summary>
		///   Copies the <see cref='WebServiceCollection'/> values to a one-dimensional <see cref='Array'/> instance at the 
		///    specified index.
		/// </summary>
		/// <param name='array'>The one-dimensional <see cref='Array'/> that is the destination of the values copied from <see cref='WebServiceCollection'/>.</param>
		/// <param name='index'>The index in <paramref name='array'/> where copying begins.</param>
		/// <exception cref='ArgumentException'>
		///   <para><paramref name='array'/> is multidimensional.</para>
		///   <para>-or-</para>
		///   <para>The number of elements in the <see cref='WebServiceCollection'/> is greater than
		///         the available space between <paramref name='arrayIndex'/> and the end of
		///         <paramref name='array'/>.</para>
		/// </exception>
		/// <exception cref='ArgumentNullException'><paramref name='array'/> is <see langword='null'/>. </exception>
		/// <exception cref='ArgumentOutOfRangeException'><paramref name='arrayIndex'/> is less than <paramref name='array'/>'s lowbound. </exception>
		/// <seealso cref='Array'/>
		public void CopyTo(WebService[] array, int index)
		{
			List.CopyTo(array, index);
		}
		
		/// <summary>
		///    <para>Returns the index of a <see cref='WebService'/> in 
		///       the <see cref='WebServiceCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='WebService'/> to locate.</param>
		/// <returns>
		/// <para>The index of the <see cref='WebService'/> of <paramref name='val'/> in the 
		/// <see cref='WebServiceCollection'/>, if found; otherwise, -1.</para>
		/// </returns>
		/// <seealso cref='WebServiceCollection.Contains'/>
		public int IndexOf(WebService val)
		{
			return List.IndexOf(val);
		}
		
		/// <summary>
		/// <para>Gets a value indicating whether the 
		///    <see cref='WebServiceCollection'/> contains the specified <see cref='WebService'/>.</para>
		/// </summary>
		/// <param name='val'>The <see cref='WebService'/> to locate.</param>
		/// <returns>
		/// <para><see langword='true'/> if the <see cref='WebService'/> is contained in the collection; 
		///   otherwise, <see langword='false'/>.</para>
		/// </returns>
		/// <seealso cref='WebServiceCollection.IndexOf'/>
		public bool Contains(WebService val)
		{
			return List.Contains(val);
		}
		
		/// <summary>
		/// <para>Inserts a <see cref='WebService'/> into the <see cref='WebServiceCollection'/> at the specified index.</para>
		/// </summary>
		/// <param name='index'>The zero-based index where <paramref name='val'/> should be inserted.</param>
		/// <param name='val'>The <see cref='WebService'/> to insert.</param>
		/// <returns><para>None.</para></returns>
		/// <seealso cref='WebServiceCollection.Add'/>
		public void Insert(int index, WebService val)
		{
			List.Insert(index, val);
		}		
		
		/// <summary>
		///   Removes a specific <see cref='WebService'/> from the <see cref='WebServiceCollection'/>.
		/// </summary>
		/// <param name='val'>The <see cref='WebService'/> to remove from the <see cref='WebServiceCollection'/>.</param>
		/// <exception cref='ArgumentException'><paramref name='val'/> is not found in the Collection.</exception>
		public void Remove(WebService val)
		{
			List.Remove(val);
		}	
		
				/// <summary>
		/// <para>Copies the elements of an array to the end of the <see cref='WebServiceCollection'/>.</para>
		/// </summary>
		/// <param name='val'>
		///    An array of type <see cref='WebService'/> containing the objects to add to the collection.
		/// </param>
		/// <returns>
		///   <para>None.</para>
		/// </returns>
		/// <seealso cref='WebServiceCollection.Add'/>
		public void AddRange(WebService[] val)
		{
			for (int i = 0; i < val.Length; i++) {
				this.Add(val[i]);
			}
		}
		
		/// <summary>
		///     <para>
		///       Adds the contents of another <see cref='WebServiceCollection'/> to the end of the collection.
		///    </para>
		/// </summary>
		/// <param name='val'>
		///    A <see cref='WebServiceCollection'/> containing the objects to add to the collection.
		/// </param>
		/// <returns>
		///   <para>None.</para>
		/// </returns>
		/// <seealso cref='WebServiceCollection.Add'/>
		public void AddRange(WebServiceCollection val)
		{
			for (int i = 0; i < val.Count; i++)
			{
				this.Add(val[i]);
			}
		}
	}
}
