// 
// SoapTrace.Net - A soap message monitoring utility. 
// 
// Copyright (C) 2004 Matthew Ward
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 
// Matthew Ward (mrward@users.sourceforge.net)
// 

using System;
using System.Diagnostics;
using System.Xml.Serialization;

namespace SoapTrace.Core.Formatters
{
	/// <summary>
	/// A web service operation.
	/// </summary>
	[ XmlType( "operation" ) ]
	public class WebServiceOperation : ICloneable
	{
		/// <summary>
		/// The name of the web service operation.
		/// </summary>
		private string m_Name = String.Empty;

		/// <summary>
		/// The web service operation's parameters.
		/// </summary>
		private WebServiceParameterCollection m_Parameters = 
			new WebServiceParameterCollection( );
		
		/// <summary>
		/// The web service return value associated with this
		/// web service operation.</summary>
		private WebServiceReturnValue m_ReturnValue =
			new WebServiceReturnValue( );

		/// <summary>
		/// Creates a new instance of the <see cref="WebServiceOperation"/>
		/// class.
		/// </summary>
		public WebServiceOperation( )
		{
		}

		/// <summary>
		/// The name of the web service operation.
		/// </summary>
		[ XmlAttribute( "name" ) ]
		public string Name
		{
			[ DebuggerStepThrough ]
			get
			{
				return m_Name;
			}

			[ DebuggerStepThrough ]
			set
			{
				m_Name = value;
			}
		}

		/// <summary>
		/// The web service operation's parameters.
		/// </summary>
		[ XmlArray( "params" ) ]
		public WebServiceParameterCollection Parameters
		{
			[ DebuggerStepThrough ]
			get
			{
				return m_Parameters;
			}
		}
		
		/// <summary>
		/// Gets or sets the return value associated with this 
		/// web service operation.
		/// </summary>
		[ XmlElement( "returnValue" ) ]
		public WebServiceReturnValue ReturnValue
		{
			get
			{
				return m_ReturnValue;
			}
			
			set
			{
				m_ReturnValue = value;	
			}
		}

		#region ICloneable Members

		/// <summary>
		/// Takes a deep copy of this class.
		/// </summary>
		/// <returns></returns>
		public object Clone( )
		{
			WebServiceOperation Operation = new WebServiceOperation( );

			Operation.Name = String.Copy( Name );

			Operation.Parameters.AddRange( ( WebServiceParameterCollection )Parameters.Clone( ) );
			
			Operation.ReturnValue = ( WebServiceReturnValue )ReturnValue.Clone( );

			return Operation;
		}

		#endregion
		
	}
}
