// 
// SoapTrace.Net - A soap message monitoring utility. 
// 
// Copyright (C) 2004 Matthew Ward
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 
// Matthew Ward (mrward@users.sourceforge.net)
// 

using System;
using System.Diagnostics;
using System.Collections;
using System.Globalization;
using System.Xml.Serialization;

namespace SoapTrace.Core.Formatters
{
	/// <summary>
	/// A type safe collection of WebServiceParameter objects.
	/// </summary>
	public class WebServiceParameterCollection : CollectionBase, ICloneable
	{
		/// <summary>
		/// Creates a new instance of the 
		/// <see cref="WebServiceParameterCollection"/> class.
		/// </summary>
		public WebServiceParameterCollection( )
		{
		}

		/// <summary>
		/// Get or sets the parameter at the specified index.
		/// </summary>
		public WebServiceParameter this[ int index ]
		{
			[ DebuggerStepThrough ]
			get
			{
				return ( WebServiceParameter )InnerList[ index ];
			}

			[ DebuggerStepThrough ]
			set
			{
				InnerList[ index ] = value; 
			}
		}

		/// <summary>
		/// Gets a parameter via its name.
		/// </summary>
		public WebServiceParameter this[ string name ]
		{
			[ DebuggerStepThrough ]
			get
			{
				return Find( name );
			}
		}

		/// <summary>
		/// Adds a web service parameter to the collection.
		/// </summary>
		/// <param name="item">The parameter to add.</param>
		public void Add( WebServiceParameter item )
		{
			InnerList.Add( item );
		}

		/// <summary>
		/// Adds a set of parameters to the collection.
		/// </summary>
		/// <param name="items"></param>
		public void Add( WebServiceParameter[ ] items )
		{
			foreach( WebServiceParameter Parameter in items )
			{
				Add( Parameter );
			}
		}

		/// <summary>
		/// Find the web service parameter via its name.
		/// </summary>
		/// <param name="name">The parameter's name.</param>
		/// <returns></returns>
		public WebServiceParameter Find( string name )
		{
			WebServiceParameter MatchedParameter = null;
	
			for( int i = 0; ( MatchedParameter == null ) && ( i < InnerList.Count ); ++i )
			{
				WebServiceParameter Parameter = ( WebServiceParameter )InnerList[ i ];
			
				if( String.Compare( Parameter.Name, name, true, CultureInfo.InvariantCulture ) == 0 )
				{
					MatchedParameter = Parameter;
				}
			}

			return MatchedParameter;
		}

		#region ICloneable Members

		/// <summary>
		/// Takes a deep copy of this object.
		/// </summary>
		/// <returns></returns>
		public object Clone( )
		{
			WebServiceParameterCollection Parameters = new WebServiceParameterCollection( );

			foreach( WebServiceParameter Parameter in this )
			{
				Parameters.Add( ( WebServiceParameter )Parameter.Clone( ) );
			}

			return Parameters;
		}

		#endregion
		
		/// <summary>
		///   Copies the <see cref='WebServiceParameterCollection'/> values to a one-dimensional <see cref='Array'/> instance at the 
		///    specified index.
		/// </summary>
		/// <param name='array'>The one-dimensional <see cref='Array'/> that is the destination of the values copied from <see cref='WebServiceParameterCollection'/>.</param>
		/// <param name='index'>The index in <paramref name='array'/> where copying begins.</param>
		/// <exception cref='ArgumentException'>
		///   <para><paramref name='array'/> is multidimensional.</para>
		///   <para>-or-</para>
		///   <para>The number of elements in the <see cref='WebServiceParameterCollection'/> is greater than
		///         the available space between <paramref name='arrayIndex'/> and the end of
		///         <paramref name='array'/>.</para>
		/// </exception>
		/// <exception cref='ArgumentNullException'><paramref name='array'/> is <see langword='null'/>. </exception>
		/// <exception cref='ArgumentOutOfRangeException'><paramref name='arrayIndex'/> is less than <paramref name='array'/>'s lowbound. </exception>
		/// <seealso cref='Array'/>
		public void CopyTo(WebServiceParameter[] array, int index)
		{
			List.CopyTo(array, index);
		}
		
		/// <summary>
		///   Removes a specific <see cref='WebServiceParameter'/> from the <see cref='WebServiceParameterCollection'/>.
		/// </summary>
		/// <param name='val'>The <see cref='WebServiceParameter'/> to remove from the <see cref='WebServiceParameterCollection'/>.</param>
		/// <exception cref='ArgumentException'><paramref name='val'/> is not found in the Collection.</exception>
		public void Remove(WebServiceParameter val)
		{
			List.Remove(val);
		}	
		
		/// <summary>
		/// <para>Inserts a <see cref='WebServiceParameter'/> into the <see cref='WebServiceParameterCollection'/> at the specified index.</para>
		/// </summary>
		/// <param name='index'>The zero-based index where <paramref name='val'/> should be inserted.</param>
		/// <param name='val'>The <see cref='WebServiceParameter'/> to insert.</param>
		/// <returns><para>None.</para></returns>
		/// <seealso cref='WebServiceParameterCollection.Add'/>
		public void Insert(int index, WebServiceParameter val)
		{
			List.Insert(index, val);
		}	
		
		/// <summary>
		///    <para>Returns the index of a <see cref='WebServiceParameter'/> in 
		///       the <see cref='WebServiceParameterCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='WebServiceParameter'/> to locate.</param>
		/// <returns>
		/// <para>The index of the <see cref='WebServiceParameter'/> of <paramref name='val'/> in the 
		/// <see cref='WebServiceParameterCollection'/>, if found; otherwise, -1.</para>
		/// </returns>
		public int IndexOf(WebServiceParameter val)
		{
			return List.IndexOf(val);
		}	
		
		/// <summary>
		/// <para>Gets a value indicating whether the 
		///    <see cref='WebServiceParameterCollection'/> contains the specified <see cref='WebServiceParameter'/>.</para>
		/// </summary>
		/// <param name='val'>The <see cref='WebServiceParameter'/> to locate.</param>
		/// <returns>
		/// <para><see langword='true'/> if the <see cref='WebServiceParameter'/> is contained in the collection; 
		///   otherwise, <see langword='false'/>.</para>
		/// </returns>
		/// <seealso cref='WebServiceParameterCollection.IndexOf'/>
		public bool Contains(WebServiceParameter val)
		{
			return List.Contains(val);
		}	
		
		/// <summary>
		/// <para>Copies the elements of an array to the end of the <see cref='WebServiceParameterCollection'/>.</para>
		/// </summary>
		/// <param name='val'>
		///    An array of type <see cref='WebServiceParameter'/> containing the objects to add to the collection.
		/// </param>
		/// <returns>
		///   <para>None.</para>
		/// </returns>
		/// <seealso cref='WebServiceParameterCollection.Add'/>
		public void AddRange(WebServiceParameter[] val)
		{
			for (int i = 0; i < val.Length; i++) {
				this.Add(val[i]);
			}
		}
		
		/// <summary>
		///     <para>
		///       Adds the contents of another <see cref='WebServiceParameterCollection'/> to the end of the collection.
		///    </para>
		/// </summary>
		/// <param name='val'>
		///    A <see cref='WebServiceParameterCollection'/> containing the objects to add to the collection.
		/// </param>
		/// <returns>
		///   <para>None.</para>
		/// </returns>
		/// <seealso cref='WebServiceParameterCollection.Add'/>
		public void AddRange(WebServiceParameterCollection val)
		{
			for (int i = 0; i < val.Count; i++)
			{
				this.Add(val[i]);
			}
		}		
	}
}
