﻿// 
// SoapTrace.Net - A soap message monitoring utility. 
// 
// Copyright (C) 2004 Matthew Ward
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 
// Matthew Ward (mrward@users.sourceforge.net)
// 

using NUnit.Framework;
using SoapTrace.Core.Communication;
using System;
using System.Text;

namespace SoapTrace.Tests
{
	/// <summary>
	/// Test fixture to test a http continue response.  This test ensures
	/// that when a http continue message is received the parser will correctly
	/// extract the http message header and body.
	/// </summary>
	[ TestFixture ]
	public class HttpContinueResponseTestFixture
	{
		/// <summary>
		/// The complete Http message response. 
		/// </summary>
		private string m_HttpMessageResponse = String.Empty;

		/// <summary>
		/// The Http header.
		/// </summary>
		private string m_HttpContinueHeader = "HTTP/1.1 100 Continue\r\nServer: Microsoft-IIS/5.0\r\nDate: Tue, 28 Sep 2004 14:54:54 GMT\r\nX-Powered-By: ASP.NET\r\n\r\n";

		/// <summary>
		/// The http header without the continue header.
		/// </summary>
		private string m_HttpHeader = "HTTP/1.1 200 OK\r\nServer: Microsoft-IIS/5.0\r\nDate: Sun, 04 Jul 2004 17:20:04 GMT\r\nX-Powered-By: ASP.NET\r\nX-AspNet-Version: 1.1.4322\r\nCache-Control: private, max-age=0\r\nContent-Type: text/xml; charset=utf-8\r\nContent-Length: 327\r\n\r\n";
		
		/// <summary>
		/// The complete http header.
		/// </summary>
		private string m_CompleteHttpHeader = String.Empty;

		/// <summary>
		/// The Http message body.
		/// </summary>
		private string m_HttpMessageBody = "<?xml version=\"1.0\" encoding=\"utf-8\"?><soap:Envelope xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\"><soap:Body><AddResponse xmlns=\"http://tempuri.org/\"><AddResult>121</AddResult></AddResponse></soap:Body></soap:Envelope>";
		
		/// <summary>
		/// The Soap response object under test.
		/// </summary>
		private SoapResponse m_Response = new SoapResponse( );

		/// <summary>
		/// Initialises this test fixture.
		/// </summary>
		[ SetUp ]
		public void Init( )
		{
			// Create the complete http message.
			m_CompleteHttpHeader = String.Concat( m_HttpContinueHeader, m_HttpHeader );
			
			m_HttpMessageResponse = String.Concat( m_CompleteHttpHeader, m_HttpMessageBody );

			// Parse the message.

			byte[ ] Bytes = UnicodeEncoding.UTF8.GetBytes( m_HttpMessageResponse );

			for( int i = 0; i < Bytes.Length; ++i )
			{
				byte MessageByte = Bytes[ i ];

				bool EndOfMessage = m_Response.AddByte( MessageByte );

				if( Bytes.Length == ( i + 1 ) )
				{
					Assert.IsTrue( EndOfMessage, "Should be at the end of the message." );
				}
				else
				{
					Assert.IsFalse( EndOfMessage, "Should not be at the end of the message." );
				}
			}
		}

		/// <summary>
		/// Test that the correct soap message received if a http continue
		/// is received by the parser first.
		/// </summary>
		[ Test ]
		public void MessageReceived( )
		{
			Assert.AreEqual( m_HttpHeader, m_Response.Header, "Soap Header is wrong." );
			Assert.AreEqual( m_HttpMessageBody, m_Response.Message, "Soap Message is wrong." );
		}
	}
}
