//
// SoapTrace.Net - A soap message monitoring utility. 
//
// Copyright (C) 2005 Matthew Ward
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Matthew Ward (mrward@users.sourceforge.net)

using NUnit.Framework;
using SoapTrace.Core.Diagnostics;
using System;
using System.IO;
using WinPcap;

namespace SoapTrace.Tests.Logging
{
	/// <summary>
	/// Tests the logging of a data packet and the reading back in of the
	/// data.
	/// </summary>
	[ TestFixture ]
	public class SinglePacketLoggedTestFixture
	{
		string RemoteServer = "200.1.1.1";
		DateTime PacketTimeReceived;
		int RemoteServerPort = 80;
		PacketLogEntryCollection LogEntries;
		PacketLogEntry LogEntry;
		Packet CreatedPacket;
		string PacketType = PacketLogEntry.IP;
		
		/// <summary>
		/// Initialises each test.
		/// </summary>
		[ SetUp ]
		public void Init( )
		{
			// Create packet.
			byte[ ] Bytes = new byte[ ] { 0x0, 0x1, 0x2, 0x3 };
			CreatedPacket = new Packet( Bytes );
			
			// Create packet log entry.
			PacketTimeReceived = new DateTime( 2000, 2, 2, 5, 15, 30, 60 );
			PacketLogEntry NewLogEntry = new PacketLogEntry( RemoteServer, RemoteServerPort, PacketTimeReceived, CreatedPacket, PacketType );

			PacketLogEntryCollection CreatedLogEntries = new PacketLogEntryCollection( );
			CreatedLogEntries.Add( NewLogEntry );
			
			// Log packet info to file.
			StringWriter StringWriter = new StringWriter( );
			PacketLogFile.Save( StringWriter, CreatedLogEntries );
			
			// Load saved packet info.			
			StringReader StringReader = new StringReader( StringWriter.ToString( ) );
			LogEntries = PacketLogFile.Load( StringReader );
			
			LogEntry = LogEntries[ 0 ];
		}
		
		/// <summary>
		/// Check we have one log entry.
		/// </summary>
		[ Test ]
		public void OneLogEntry( )
		{
			Assert.AreEqual( 1, LogEntries.Count, "Expected only one log entry." );
		}
		
		/// <summary>
		/// Checks the date time.
		/// </summary>
		[ Test ]
		public void LogEntryDateTime( )
		{
			Assert.AreEqual( PacketTimeReceived, LogEntry.PacketReceivedTime, 
			                "Log entry's packet datetime is wrong." );
		}
		
		/// <summary>
		/// Checks the remote server filter value.
		/// </summary>
		[ Test ]
		public void RemoteServerFilter( )
		{
			Assert.AreEqual( RemoteServer, LogEntry.RemoteServerFilter, 
			               "Log entry's remote server value is wrong." );
		}
		
		/// <summary>
		/// Checks the remote server port filter value.
		/// </summary>
		[ Test ]
		public void RemoteServerPortFilter( )
		{
			Assert.AreEqual( RemoteServerPort, LogEntry.RemoteServerPortFilter, 
			               "Log entry's remote server port value is wrong." );
		}
		
		/// <summary>
		/// Checks the packet type.
		/// </summary>
		[ Test ]
		public void LogEntryPacketType( )
		{
			Assert.AreEqual( PacketType, LogEntry.PacketType, 
			               "Log entry's packet type value is wrong." );
		}
		
		/// <summary>
		/// Checks the packet has been read back in correctly.
		/// </summary>
		[ Test ]
		public void LogEntryPacketByteLength( )
		{
			byte[ ] PacketBytes = LogEntry.Packet.GetBytes( );
			byte[ ] ExpectedBytes = CreatedPacket.GetBytes( );
			
			Assert.AreEqual( ExpectedBytes.Length, PacketBytes.Length, "Number of packet bytes differ." );
		}
		
		/// <summary>
		/// Checks the packet has been read back in correctly.
		/// </summary>
		[ Test ]
		public void LogEntryPacketBytes( )
		{
			byte[ ] PacketBytes = LogEntry.Packet.GetBytes( );
			byte[ ] ExpectedBytes = CreatedPacket.GetBytes( );
			
			for( int i = 0; i < ExpectedBytes.Length; ++i )
			{
				byte ActualByte = PacketBytes[ i ];
				byte ExpectedByte = ExpectedBytes[ i ];
				
				Assert.AreEqual( ExpectedByte, ActualByte, 
				                String.Concat( "Byte ", i.ToString( ), " in the packet is incorrect." ) );
			}	
		}
	}
}
