//
// SoapTrace.Net - A soap message monitoring utility. 
//
// Copyright (C) 2005 Matthew Ward
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Matthew Ward (mrward@users.sourceforge.net)


using NUnit.Framework;
using SoapTrace.Core.Communication;
using SoapTrace.Tests.Utils;
using System;
using WinPcap;

namespace SoapTrace.Tests
{
	/// <summary>
	/// Tests the <see cref="TcpPacketFilter"/> class.
	/// </summary>
	[ TestFixture ]
	public class PacketFilterTestFixture
	{
		TcpPacketFilter PacketFilter;
		string IPAddress = "200.1.1.2";
		int Port = 80;
		
		/// <summary>
		/// Initialises the test fixture.
		/// </summary>
		[ TestFixtureSetUp ]
		public void Init( )
		{
			PacketFilter = new TcpPacketFilter( IPAddress, Port );
		}
		
		/// <summary>
		/// Tests that a packet sent to the server on the correct port 
		/// passes the filter.
		/// </summary>
		[ Test ]
		public void PacketSentToServer( )
		{
			string Data = 
				"0000  00 90 F5 2B 8D 2F 00 40  F4 8C AA 2F 08 00 45 00 \n" +
				"0010  00 30 45 C0 40 00 80 06  23 02 C8 01 01 01 C8 01 \n" +
				"0020  01 02 04 4A 00 50 D0 22  05 0A 00 00 00 00 70 02 \n" +
				"0030  FF FF FC 48 00 00 02 04  05 B4 01 01 04 02 ";
			
			byte[ ] Bytes = PacketBytes.GetBytesFromString( Data );
			
			Packet Packet = new Packet( Bytes );
			
			TcpPacket TcpPacket = PacketFilter.GetTcpPacket( Packet );
			
			Assert.IsNotNull( TcpPacket, "Should have a packet." );
			
			// Source IP
			Assert.AreEqual( "200.1.1.1:1098",
				TcpPacket.SourceAddress.ToString( ),
				"Source IP address mismatch." );

			// Destination IP.

			Assert.AreEqual( "200.1.1.2:80", 
				TcpPacket.DestinationAddress.ToString( ),
				"Source IP address mismatch." );
		}
		
		/// <summary>
		/// Tests that a packet sent from the server on the correct port 
		/// passes the filter.
		/// </summary>
		[ Test ]
		public void PacketSentFromServer( )
		{
			string Data =
				"0000  00 90 F5 2B 8D 2F 00 40  F4 8C AA 2F 08 00 45 00 \n" +
				"0010  00 30 45 C0 40 00 80 06  23 02 C8 01 01 02 C8 01 \n" +
				"0020  01 01 00 50 04 4A D0 22  05 0A 00 00 00 00 70 02 \n" +
				"0030  FF FF FC 48 00 00 02 04  05 B4 01 01 04 02 ";

			byte[ ] Bytes = PacketBytes.GetBytesFromString( Data );
			
			Packet Packet = new Packet( Bytes );
			
			TcpPacket TcpPacket = PacketFilter.GetTcpPacket( Packet );
			
			Assert.IsNotNull( TcpPacket, "Should have a packet." );
			
			// Source IP
			Assert.AreEqual( "200.1.1.2:80",
				TcpPacket.SourceAddress.ToString( ),
				"Source IP address mismatch." );

			// Destination IP.

			Assert.AreEqual( "200.1.1.1:1098", 
				TcpPacket.DestinationAddress.ToString( ),
				"Source IP address mismatch." );
		}
		
		/// <summary>
		/// Tests that a packet sent to the server on the incorrect port 
		/// is rejected by the filter.
		/// </summary>
		[ Test ]
		public void PacketSentToDifferentServerPort( )
		{
			string Data =
				"0000  00 90 F5 2B 8D 2F 00 40  F4 8C AA 2F 08 00 45 00 \n" +
				"0010  00 30 45 C0 40 00 80 06  23 02 C8 01 01 01 C8 01 \n" +
				"0020  01 02 04 4A 00 51 D0 22  05 0A 00 00 00 00 70 02 \n" +
				"0030  FF FF FC 48 00 00 02 04  05 B4 01 01 04 02 ";

			byte[ ] Bytes = PacketBytes.GetBytesFromString( Data );
			
			Packet Packet = new Packet( Bytes );
			
			TcpPacket TcpPacket = PacketFilter.GetTcpPacket( Packet );
			
			Assert.IsNull( TcpPacket, "Should not have a packet." );
		}		
		
		/// <summary>
		/// Tests that a packet sent to the wrong IP address on the correct port 
		/// is rejected by the filter.
		/// </summary>
		[ Test ]
		public void PacketSentToDifferentServer( )
		{
			string Data =
				"0000  00 90 F5 2B 8D 2F 00 40  F4 8C AA 2F 08 00 45 00 \n" +
				"0010  00 30 45 C0 40 00 80 06  23 02 C8 01 01 01 C8 01 \n" +
				"0020  01 03 04 4A 00 50 D0 22  05 0A 00 00 00 00 70 02 \n" +
				"0030  FF FF FC 48 00 00 02 04  05 B4 01 01 04 02 ";

			byte[ ] Bytes = PacketBytes.GetBytesFromString( Data );
			
			Packet Packet = new Packet( Bytes );
			
			TcpPacket TcpPacket = PacketFilter.GetTcpPacket( Packet );
			
			Assert.IsNull( TcpPacket, "Should not have a packet." );
		}				
		
		/// <summary>
		/// Tests that a non-TCP packet is rejected.
		/// </summary>
		[ Test ]
		public void NonTcpPacket( )
		{
			string Data =
				"0000  00 90 F5 2B 8D 2F 00 40  F4 8C AA 2F 08 00 45 00 \n" +
				"0010  00 30 45 C0 40 00 80 04  23 02 C8 01 01 01 C8 01 \n" +
				"0020  01 03 04 4A 00 50 D0 22  05 0A 00 00 00 00 70 02 \n" +
				"0030  FF FF FC 48 00 00 02 04  05 B4 01 01 04 02 ";

			byte[ ] Bytes = PacketBytes.GetBytesFromString( Data );
			
			Packet Packet = new Packet( Bytes );
			
			TcpPacket TcpPacket = PacketFilter.GetTcpPacket( Packet );
			
			Assert.IsNull( TcpPacket, "Should not have a packet." );
		}
	}
}
