//
// SoapTrace.Net - A soap message monitoring utility. 
//
// Copyright (C) 2005 Matthew Ward
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Matthew Ward (mrward@users.sourceforge.net)

using NUnit.Framework;
using SoapTrace.Core.Communication;
using SoapTrace.Core.Diagnostics;
using System;
using System.IO;
using System.Text;

namespace SoapTrace.Tests.TcpPacketMonitoring
{
	/// <summary>
	/// Tests that the <see cref="TcpPacketMonitor"/> class's
	/// handling of a missed tcp segment.
	/// </summary>
	[ TestFixture ]
	public class MissedOneResponseSegmentTestFixture
	{
		TcpPacketMonitor Monitor;
		StringBuilder ClientMessage;
		StringBuilder ServerMessage;
		
		/// <summary>
		/// Initialises each test.
		/// </summary>
		[ SetUp ]
		public void Init( )
		{
			ClientMessage = new StringBuilder( );
			ServerMessage = new StringBuilder( );
			
			Monitor = new TcpPacketMonitor( "209.171.52.99", 80 );
			Monitor.MissedConnectionBytesTransferred += new TcpCommunicationEventHandler( OnBytesTransferred );
			
			StringReader Reader = new StringReader( GetPacketXml( ) );
			PacketLogEntryCollection LogEntries = PacketLogFile.Load( Reader );
			
			foreach( PacketLogEntry Entry in LogEntries )
			{
				Monitor.ProcessPacket( Entry.Packet );
			}
		}
			
		/// <summary>
		/// Check we received the correct message.
		/// </summary>
		[ Test ]
		public void ClientMessageSent( )
		{
			string ExpectedMessage = "POST /webservices/latest.asmx HTTP/1.1\r\n" +
				"User-Agent: Mozilla/4.0 (compatible; MSIE 6.0; MS Web Services Client Protocol 1.1.4322.573)\r\n" +
				"Content-Type: text/xml; charset=utf-8\r\n" +
				"SOAPAction: \"http://codeproject.com/webservices/GetLatestArticleBrief\"\r\n" +
				"Content-Length: 366\r\n" +
				"Expect: 100-continue\r\n" +
				"Host: www.codeproject.com\r\n" +
				"\r\n" +
				"<?xml version=\"1.0\" encoding=\"utf-8\"?>" +
				"<soap:Envelope xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\">" +
				"<soap:Body>" +
				"<GetLatestArticleBrief xmlns=\"http://codeproject.com/webservices/\">" +
				"<NumArticles>10</NumArticles>" +
				"</GetLatestArticleBrief>" +
				"</soap:Body>" +
				"</soap:Envelope>";
			
			Assert.AreEqual( ExpectedMessage, ClientMessage.ToString( ), "Unexpected message received." );
		}
		
		
		/// <summary>
		/// Check we received the correct message.
		/// </summary>
		[ Test ]
		public void ServerMessageReceived( )
		{
			string ExpectedMessage = "HTTP/1.1 100 Continue\r\n" +
				"Server: Microsoft-IIS/5.0\r\n" +
				"Date: Thu, 10 Feb 2005 20:12:05 GMT\r\n" +
				"X-Powered-By: ASP.NET\r\n" +
				"\r\n" +
				"HTTP/1.1 200 OK\r\n" +
				"Server: Microsoft-IIS/5.0\r\n" +
				"Date: Thu, 10 Feb 2005 20:12:05 GMT\r\n" +
				"X-Powered-By: ASP.NET\r\n" +
				"X-AspNet-Version: 1.1.4322\r\n" +
				"Cache-Control: no-cache\r\n" +
				"Pragma: no-cache\r\n" +
				"Expires: -1\r\n" +
				"Content-Type: text/xml; charset=utf-8\r\n" +
				"Content-Length: 5284\r\n" +
				"\r\n" +
				"<?xml version=\"1.0\" encoding=\"utf-8\"?>" +
				"<soap:Envelope xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\">" +
				"<soap:Body>" +
				"<GetLatestArticleBriefResponse xmlns=\"http://codeproject.com/webservices/\">" +
				"<GetLatestArticleBriefResult>" +
				"<ArticleBrief>" +
				"<URL>http://www.codeproject.com/useritems/Screen_Capture_Form.asp</URL>" +
				"<Title>Screen Capture a VB.Net Form</Title>" +
				"<Author>scoplan</Author><Posted>2005-02-10T13:08:00.0000000-05:00</Posted>" +
				"<Updated>2005-02-10T13:08:00.0000000-05:00</Updated>" +
				"<Status>Available New</Status>" +
				"<Description>Screen Capture a VB.Net Form (or any object with an Hdc)  to the clipboard (or file); Sections: .NET; Subsection: Unedited Reader Contributions)</Description></ArticleBrief>" +
				"<ArticleBrief><URL>http://www.codeproject.com/useritems/ProxyFtp.asp</URL><Title>How Proxy Server serves FTP clients?</Title><Author>HatemMostafa</Author><Posted>2005-02-10T11:39:00.0000000-05:00</Posted><Updated>2005-02-10T11:39:00.0000000-05:00</Updated><Status>Available New</Status><Description>This artivle describes how Proxy Server deals with PORT and PASV FTP commands; Sections: C++ / MFC; Subsection: Unedited Reader Contributions)</Description></ArticleBrief>" +
				"<ArticleBrief><URL>http://www.codeproject.com/useritems/madxlib.asp</URL><Title>DLL For Buffer-based MP3 Decoding</Title><Author>gilad-ap</Author><Posted>2005-02-04T12:57:00.0000000-05:00</Posted><Updated>2005-02-10T10:55:00.0000000-05:00</Updated><Status>Available New Updated</Status><Description>Source code to produce a DLL that converts MP3 files to PCM using a buffer-based mechanism. Based on the open source library libmad.; Sections: Audio and Video; Subsection: Unedited Reader Contributions)</Description></ArticleBrief>" +
				"<ArticleBrief><URL>http://www.codeproject.com/useritems/GetLoggingWithEntLib.asp</URL><Title>Get Logging with the Enterprise Library</Title><Author>Piers Lawson</Author><Posted>2005-02-10T10:06:00.0000000-05:00</Posted><Updated>2005-02-10T10:06:00.0000000-05:00</Updated><Status>Available New</Status><Description>How to get the basics of the Enterprise Library's Logging and Instrumentation Application Block working with your application.; Sections: .NET; Subsection: Unedited Reader Contributions)</Description></ArticleBrief>" +
				"<ArticleBrief><URL>http://www.codeproject.com/useritems/Incremental_search.asp</URL><Title>How to implements an incremental search using a grid (UltraWinGrid)</Title><Author>CFQeb</Author><Posted>2005-02-10T09:47:00.0000000-05:00</Posted><Updated>2005-02-10T09:47:00.0000000-05:00</Updated><Status>Available New</Status><Description>This article pretends show how implement an incremental search doing double click in a UltraWinGrid (the code must works fine in an implicit grid); Sections: VB.NET; Subsection: Unedited Reader Contributions)</Description></ArticleBrief>" +
				"<ArticleBrief><URL>http://www.codeproject.com/useritems/Heybo_Message_Box.asp</URL><Title>Heybo Message Box-Customizable Message Box</Title><Author>hmlinder</Author><Posted>2005-02-10T09:41:00.0000000-05:00</Posted><Updated>2005-02-10T09:41:00.0000000-05:00</Updated><Status>Available New</Status><Description>A customizable MessageBox with sound; Sections: C# Controls; Subsection: Unedited Reader Contributions)</Description>" +
				"</ArticleBrief><ArticleBrief><URL>http://www.codeproject.com/csharp/ArticleSubZero.asp</URL><Title>Article SubZero: CaseBase, A Testing Framework for C#</Title><Author>Tom Ollar, Jim Bennett</Author><Posted>2005-01-27T16:20:00.0000000-05:00</Posted><Updated>2005-02-10T05:23:00.0000000-05:00</Updated><Status>Available New Updated Moved</Status><Description>Describes a testing framework for use with Visual Studio and C#.; Sections: C# Programming; Subsection: Debug and Tracing)</Description></ArticleBrief>" +
				"<ArticleBrief><URL>http://www.codeproject.com/cs/miscctrl/RichTextBoxExtended.asp</URL><Title>RichTextBoxExtended</Title><Author>Richard Parsons</Author><Posted>2005-02-10T04:52:00.0000000-05:00</Posted><Updated>2005-02-10T04:52:00.0000000-05:00</Updated><Status>Available New</Status><Description>A RichTextBox with a RichText toolbar.; Sections: C# Controls; Subsection: Edit Controls)</Description></ArticleBrief><ArticleBrief><URL>http://www.codeproject.com/file/FindWhen.asp</URL><Title>FindWhen</Title><Author>Roger65</Author><Posted>2005-02-10T03:59:00.0000000-05:00</Posted><Updated>2005-02-10T03:59:00.0000000-05:00</Updated><Status>Available New</Status><Description>An article on finding files created before a date.; Sections: Files and Folders; Subsection: General)</Description></ArticleBrief>" +
				"<ArticleBrief><URL>http://www.codeproject.com/richedit/COleRichEditCtrl.asp</URL><Title>A Rich Edit Control That Displays Bitmaps and Other OLE Objects</Title><Author>Mike O'Neill</Author><Posted>2005-02-10T03:11:00.0000000-05:00</Posted><Updated>2005-02-10T03:11:00.0000000-05:00</Updated><Status>Available New</Status><Description>COleRichEditCtrl will display RTF text as well as bitmaps, video clips, Word, Excel and PowerPoint documents, and any other kind of OLE objects.; Sections: Rich Edit Control; Subsection: General)</Description></ArticleBrief>" +
				"</GetLatestArticleBriefResult></GetLatestArticleBriefResponse></soap:Body></soap:Envelope>";
			
			Assert.AreEqual( ExpectedMessage, ServerMessage.ToString( ), "Unexpected message received." );
		}
		
		private void OnBytesTransferred( object sender, TcpCommunicationEventArgs e )
		{
			if( "209.171.52.99:80" == e.SourceAddress.ToString( ) )
			{
				// Server message.
				ServerMessage.Append( UnicodeEncoding.UTF8.GetString( e.GetBytes( ) ) );
			}
			else
			{
				// Client message.
				ClientMessage.Append( UnicodeEncoding.UTF8.GetString( e.GetBytes( ) ) );
			}
		}
		
		/// <summary>
		/// Gets the xml to be read in by the <see cref="PacketLogFile"/>
		/// class.
		/// </summary>
		private string GetPacketXml( )
		{
			return "<Packets>\r\n" +
				"  <Packet Time=\"2005-02-10 20:22:15.58\" ServerFilter=\"209.171.52.99\" PortFilter=\"80\" PacketType=\"Ethernet\">\r\n" +
				"    <Bytes>00 11 F5 0B 1F 69 00 11 F5 0B 1F 6A 08 00 45 00 01 65 DE 8E 40 00 80 06 53 49 C0 A8 01 04 D1 AB 34 63 09 E6 00 50 93 B5 4F 26 93 6D 2B 26 50 18 FB 04 8F 30 00 00 50 4F 53 54 20 2F 77 65 62 73 65 72 76 69 63 65 73 2F 6C 61 74 65 73 74 2E 61 73 6D 78 20 48 54 54 50 2F 31 2E 31 0D 0A 55 73 65 72 2D 41 67 65 6E 74 3A 20 4D 6F 7A 69 6C 6C 61 2F 34 2E 30 20 28 63 6F 6D 70 61 74 69 62 6C 65 3B 20 4D 53 49 45 20 36 2E 30 3B 20 4D 53 20 57 65 62 20 53 65 72 76 69 63 65 73 20 43 6C 69 65 6E 74 20 50 72 6F 74 6F 63 6F 6C 20 31 2E 31 2E 34 33 32 32 2E 35 37 33 29 0D 0A 43 6F 6E 74 65 6E 74 2D 54 79 70 65 3A 20 74 65 78 74 2F 78 6D 6C 3B 20 63 68 61 72 73 65 74 3D 75 74 66 2D 38 0D 0A 53 4F 41 50 41 63 74 69 6F 6E 3A 20 22 68 74 74 70 3A 2F 2F 63 6F 64 65 70 72 6F 6A 65 63 74 2E 63 6F 6D 2F 77 65 62 73 65 72 76 69 63 65 73 2F 47 65 74 4C 61 74 65 73 74 41 72 74 69 63 6C 65 42 72 69 65 66 22 0D 0A 43 6F 6E 74 65 6E 74 2D 4C 65 6E 67 74 68 3A 20 33 36 36 0D 0A 45 78 70 65 63 74 3A 20 31 30 30 2D " +
				"63 6F 6E 74 69 6E 75 65 0D 0A 48 6F 73 74 3A 20 77 77 77 2E 63 6F 64 65 70 72 6F 6A 65 63 74 2E 63 6F 6D 0D 0A 0D 0A </Bytes>\r\n" +
				"  </Packet>\r\n" +
				"  <Packet Time=\"2005-02-10 20:22:16.15\" ServerFilter=\"209.171.52.99\" PortFilter=\"80\" PacketType=\"Ethernet\">\r\n" +
				"    <Bytes>00 11 F5 0B 1F 6A 00 11 F5 0B 1F 69 08 00 45 00 00 98 6B FA 40 00 66 06 E0 AA D1 AB 34 63 C0 A8 01 04 00 50 09 E6 93 6D 2B 26 93 B5 50 63 50 18 FB FF 4F 87 00 00 48 54 54 50 2F 31 2E 31 20 31 30 30 20 43 6F 6E 74 69 6E 75 65 0D 0A 53 65 72 76 65 72 3A 20 4D 69 63 72 6F 73 6F 66 74 2D 49 49 53 2F 35 2E 30 0D 0A 44 61 74 65 3A 20 54 68 75 2C 20 31 30 20 46 65 62 20 32 30 30 35 20 32 30 3A 31 32 3A 30 35 20 47 4D 54 0D 0A 58 2D 50 6F 77 65 72 65 64 2D 42 79 3A 20 41 53 50 2E 4E 45 54 0D 0A 0D 0A </Bytes>\r\n" +
				"  </Packet>\r\n" +
				"  <Packet Time=\"2005-02-10 20:22:16.20\" ServerFilter=\"209.171.52.99\" PortFilter=\"80\" PacketType=\"Ethernet\">\r\n" +
				"    <Bytes>00 11 F5 0B 1F 69 00 11 F5 0B 1F 6A 08 00 45 00 01 96 DE 8F 40 00 80 06 53 17 C0 A8 01 04 D1 AB 34 63 09 E6 00 50 93 B5 50 63 93 6D 2B 96 50 18 FA 94 CF 8E 00 00 3C 3F 78 6D 6C 20 76 65 72 73 69 6F 6E 3D 22 31 2E 30 22 20 65 6E 63 6F 64 69 6E 67 3D 22 75 74 66 2D 38 22 3F 3E 3C 73 6F 61 70 3A 45 6E 76 65 6C 6F 70 65 20 78 6D 6C 6E 73 3A 73 6F 61 70 3D 22 68 74 74 70 3A 2F 2F 73 63 68 65 6D 61 73 2E 78 6D 6C 73 6F 61 70 2E 6F 72 67 2F 73 6F 61 70 2F 65 6E 76 65 6C 6F 70 65 2F 22 20 78 6D 6C 6E 73 3A 78 73 69 3D 22 68 74 74 70 3A 2F 2F 77 77 77 2E 77 33 2E 6F 72 67 2F 32 30 30 31 2F 58 4D 4C 53 63 68 65 6D 61 2D 69 6E 73 74 61 6E 63 65 22 20 78 6D 6C 6E 73 3A 78 73 64 3D 22 68 74 74 70 3A 2F 2F 77 77 77 2E 77 33 2E 6F 72 67 2F 32 30 30 31 2F 58 4D 4C 53 63 68 65 6D 61 22 3E 3C 73 6F 61 70 3A 42 6F 64 79 3E 3C 47 65 74 4C 61 74 65 73 74 41 72 74 69 63 6C 65 42 72 69 65 66 20 78 6D 6C 6E 73 3D 22 68 74 74 70 3A 2F 2F 63 6F 64 65 70 72 6F 6A 65 63 74 2E 63 6F 6D 2F 77 65 62 73 65 72 76 " +
				"69 63 65 73 2F 22 3E 3C 4E 75 6D 41 72 74 69 63 6C 65 73 3E 31 30 3C 2F 4E 75 6D 41 72 74 69 63 6C 65 73 3E 3C 2F 47 65 74 4C 61 74 65 73 74 41 72 74 69 63 6C 65 42 72 69 65 66 3E 3C 2F 73 6F 61 70 3A 42 6F 64 79 3E 3C 2F 73 6F 61 70 3A 45 6E 76 65 6C 6F 70 65 3E </Bytes>\r\n" +
				"  </Packet>\r\n" +
				"  <Packet Time=\"2005-02-10 20:22:16.26\" ServerFilter=\"209.171.52.99\" PortFilter=\"80\" PacketType=\"Ethernet\">\r\n" +
				"    <Bytes>00 11 F5 0B 1F 6A 00 11 F5 0B 1F 69 08 00 45 00 05 B4 6C 0B 40 00 66 06 DB 7D D1 AB 34 63 C0 A8 01 04 00 50 09 E6 93 6D 2B 96 93 B5 51 D1 50 10 FA 91 8E 37 00 00 48 54 54 50 2F 31 2E 31 20 32 30 30 20 4F 4B 0D 0A 53 65 72 76 65 72 3A 20 4D 69 63 72 6F 73 6F 66 74 2D 49 49 53 2F 35 2E 30 0D 0A 44 61 74 65 3A 20 54 68 75 2C 20 31 30 20 46 65 62 20 32 30 30 35 20 32 30 3A 31 32 3A 30 35 20 47 4D 54 0D 0A 58 2D 50 6F 77 65 72 65 64 2D 42 79 3A 20 41 53 50 2E 4E 45 54 0D 0A 58 2D 41 73 70 4E 65 74 2D 56 65 72 73 69 6F 6E 3A 20 31 2E 31 2E 34 33 32 32 0D 0A 43 61 63 68 65 2D 43 6F 6E 74 72 6F 6C 3A 20 6E 6F 2D 63 61 63 68 65 0D 0A 50 72 61 67 6D 61 3A 20 6E 6F 2D 63 61 63 68 65 0D 0A 45 78 70 69 72 65 73 3A 20 2D 31 0D 0A 43 6F 6E 74 65 6E 74 2D 54 79 70 65 3A 20 74 65 78 74 2F 78 6D 6C 3B 20 63 68 61 72 73 65 74 3D 75 74 66 2D 38 0D 0A 43 6F 6E 74 65 6E 74 2D 4C 65 6E 67 74 68 3A 20 35 32 38 34 0D 0A 0D 0A 3C 3F 78 6D 6C 20 76 65 72 73 69 6F 6E 3D 22 31 2E 30 22 20 65 6E 63 6F 64 69 6E " +
				"67 3D 22 75 74 66 2D 38 22 3F 3E 3C 73 6F 61 70 3A 45 6E 76 65 6C 6F 70 65 20 78 6D 6C 6E 73 3A 73 6F 61 70 3D 22 68 74 74 70 3A 2F 2F 73 63 68 65 6D 61 73 2E 78 6D 6C 73 6F 61 70 2E 6F 72 67 2F 73 6F 61 70 2F 65 6E 76 65 6C 6F 70 65 2F 22 20 78 6D 6C 6E 73 3A 78 73 69 3D 22 68 74 74 70 3A 2F 2F 77 77 77 2E 77 33 2E 6F 72 67 2F 32 30 30 31 2F 58 4D 4C 53 63 68 65 6D 61 2D 69 6E 73 74 61 6E 63 65 22 20 78 6D 6C 6E 73 3A 78 73 64 3D 22 68 74 74 70 3A 2F 2F 77 77 77 2E 77 33 2E 6F 72 67 2F 32 30 30 31 2F 58 4D 4C 53 63 68 65 6D 61 22 3E 3C 73 6F 61 70 3A 42 6F 64 79 3E 3C 47 65 74 4C 61 74 65 73 74 41 72 74 69 63 6C 65 42 72 69 65 66 52 65 73 70 6F 6E 73 65 20 78 6D 6C 6E 73 3D 22 68 74 74 70 3A 2F 2F 63 6F 64 65 70 72 6F 6A 65 63 74 2E 63 6F 6D 2F 77 65 62 73 65 72 76 69 63 65 73 2F 22 3E 3C 47 65 74 4C 61 74 65 73 74 41 72 74 69 63 6C 65 42 72 69 65 66 52 65 73 75 6C 74 3E 3C 41 72 74 69 63 6C 65 42 72 69 65 66 3E 3C 55 52 4C 3E 68 74 74 70 3A 2F 2F 77 77 77 2E 63 6F 64 65 70 72 6F 6A 65 63 74 " +
				"2E 63 6F 6D 2F 75 73 65 72 69 74 65 6D 73 2F 53 63 72 65 65 6E 5F 43 61 70 74 75 72 65 5F 46 6F 72 6D 2E 61 73 70 3C 2F 55 52 4C 3E 3C 54 69 74 6C 65 3E 53 63 72 65 65 6E 20 43 61 70 74 75 72 65 20 61 20 56 42 2E 4E 65 74 20 46 6F 72 6D 3C 2F 54 69 74 6C 65 3E 3C 41 75 74 68 6F 72 3E 73 63 6F 70 6C 61 6E 3C 2F 41 75 74 68 6F 72 3E 3C 50 6F 73 74 65 64 3E 32 30 30 35 2D 30 32 2D 31 30 54 31 33 3A 30 38 3A 30 30 2E 30 30 30 30 30 30 30 2D 30 35 3A 30 30 3C 2F 50 6F 73 74 65 64 3E 3C 55 70 64 61 74 65 64 3E 32 30 30 35 2D 30 32 2D 31 30 54 31 33 3A 30 38 3A 30 30 2E 30 30 30 30 30 30 30 2D 30 35 3A 30 30 3C 2F 55 70 64 61 74 65 64 3E 3C 53 74 61 74 75 73 3E 41 76 61 69 6C 61 62 6C 65 20 4E 65 77 3C 2F 53 74 61 74 75 73 3E 3C 44 65 73 63 72 69 70 74 69 6F 6E 3E 53 63 72 65 65 6E 20 43 61 70 74 75 72 65 20 61 20 56 42 2E 4E 65 74 20 46 6F 72 6D 20 28 6F 72 20 61 6E 79 20 6F 62 6A 65 63 74 20 77 69 74 68 20 61 6E 20 48 64 63 29 20 20 74 6F 20 74 68 65 20 63 6C 69 70 62 6F 61 72 64 20 28 6F 72 20 66 " +
				"69 6C 65 29 3B 20 53 65 63 74 69 6F 6E 73 3A 20 2E 4E 45 54 3B 20 53 75 62 73 65 63 74 69 6F 6E 3A 20 55 6E 65 64 69 74 65 64 20 52 65 61 64 65 72 20 43 6F 6E 74 72 69 62 75 74 69 6F 6E 73 29 3C 2F 44 65 73 63 72 69 70 74 69 6F 6E 3E 3C 2F 41 72 74 69 63 6C 65 42 72 69 65 66 3E 3C 41 72 74 69 63 6C 65 42 72 69 65 66 3E 3C 55 52 4C 3E 68 74 74 70 3A 2F 2F 77 77 77 2E 63 6F 64 65 70 72 6F 6A 65 63 74 2E 63 6F 6D 2F 75 73 65 72 69 74 65 6D 73 2F 50 72 6F 78 79 46 74 70 2E 61 73 70 3C 2F 55 52 4C 3E 3C 54 69 74 6C 65 3E 48 6F 77 20 50 72 6F 78 79 20 53 65 72 76 65 72 20 73 65 72 76 65 73 20 46 54 50 20 63 6C 69 65 6E 74 73 3F 3C 2F 54 69 74 6C 65 3E 3C 41 75 74 68 6F 72 3E 48 61 74 65 6D 4D 6F 73 74 61 66 61 3C 2F 41 75 74 68 6F 72 3E 3C 50 6F 73 74 65 64 3E 32 30 30 35 2D 30 32 2D 31 30 54 31 31 3A 33 39 3A 30 30 2E 30 30 30 30 30 30 30 2D 30 35 3A 30 30 3C 2F 50 6F 73 74 65 64 3E 3C 55 70 64 61 74 65 64 3E 32 30 30 35 2D 30 32 2D 31 30 54 31 31 3A 33 39 3A 30 30 2E 30 30 30 30 30 30 30 2D 30 35 " +
				"3A 30 30 3C 2F 55 70 64 61 74 65 64 3E 3C 53 74 61 74 75 73 3E 41 76 61 69 6C 61 62 6C 65 20 4E 65 77 3C 2F 53 74 61 74 75 73 3E 3C 44 65 73 63 72 69 70 74 69 6F 6E 3E 54 68 69 73 20 61 72 74 69 76 6C 65 20 64 65 73 63 72 69 62 65 73 20 68 6F 77 20 50 72 6F 78 79 20 53 65 72 76 65 72 20 64 65 61 6C 73 20 77 69 74 68 20 50 4F 52 54 20 61 6E 64 20 50 41 53 56 20 46 54 50 20 63 6F 6D 6D 61 6E 64 73 3B </Bytes>\r\n" +
				"  </Packet>\r\n" +
				"  <Packet Time=\"2005-02-10 20:22:16.31\" ServerFilter=\"209.171.52.99\" PortFilter=\"80\" PacketType=\"Ethernet\">\r\n" +
				"    <Bytes>00 11 F5 0B 1F 6A 00 11 F5 0B 1F 69 08 00 45 00 05 B4 6C 0D 40 00 66 06 DB 7B D1 AB 34 63 C0 A8 01 04 00 50 09 E6 93 6D 36 AE 93 B5 51 D1 50 10 FA 91 4B 85 00 00 30 2E 30 30 30 30 30 30 30 2D 30 35 3A 30 30 3C 2F 55 70 64 61 74 65 64 3E 3C 53 74 61 74 75 73 3E 41 76 61 69 6C 61 62 6C 65 20 4E 65 77 3C 2F 53 74 61 74 75 73 3E 3C 44 65 73 63 72 69 70 74 69 6F 6E 3E 54 68 69 73 20 61 72 74 69 63 6C 65 20 70 72 65 74 65 6E 64 73 20 73 68 6F 77 20 68 6F 77 20 69 6D 70 6C 65 6D 65 6E 74 20 61 6E 20 69 6E 63 72 65 6D 65 6E 74 61 6C 20 73 65 61 72 63 68 20 64 6F 69 6E 67 20 64 6F 75 62 6C 65 20 63 6C 69 63 6B 20 69 6E 20 61 20 55 6C 74 72 61 57 69 6E 47 72 69 64 20 28 74 68 65 20 63 6F 64 65 20 6D 75 73 74 20 77 6F 72 6B 73 20 66 69 6E 65 20 69 6E 20 61 6E 20 69 6D 70 6C 69 63 69 74 20 67 72 69 64 29 3B 20 53 65 63 74 69 6F 6E 73 3A 20 56 42 2E 4E 45 54 3B 20 53 75 62 73 65 63 74 69 6F 6E 3A 20 55 6E 65 64 69 74 65 64 20 52 65 61 64 65 72 20 43 6F 6E 74 72 69 62 75 74 69 6F 6E 73 29 3C 2F " +
				"44 65 73 63 72 69 70 74 69 6F 6E 3E 3C 2F 41 72 74 69 63 6C 65 42 72 69 65 66 3E 3C 41 72 74 69 63 6C 65 42 72 69 65 66 3E 3C 55 52 4C 3E 68 74 74 70 3A 2F 2F 77 77 77 2E 63 6F 64 65 70 72 6F 6A 65 63 74 2E 63 6F 6D 2F 75 73 65 72 69 74 65 6D 73 2F 48 65 79 62 6F 5F 4D 65 73 73 61 67 65 5F 42 6F 78 2E 61 73 70 3C 2F 55 52 4C 3E 3C 54 69 74 6C 65 3E 48 65 79 62 6F 20 4D 65 73 73 61 67 65 20 42 6F 78 2D 43 75 73 74 6F 6D 69 7A 61 62 6C 65 20 4D 65 73 73 61 67 65 20 42 6F 78 3C 2F 54 69 74 6C 65 3E 3C 41 75 74 68 6F 72 3E 68 6D 6C 69 6E 64 65 72 3C 2F 41 75 74 68 6F 72 3E 3C 50 6F 73 74 65 64 3E 32 30 30 35 2D 30 32 2D 31 30 54 30 39 3A 34 31 3A 30 30 2E 30 30 30 30 30 30 30 2D 30 35 3A 30 30 3C 2F 50 6F 73 74 65 64 3E 3C 55 70 64 61 74 65 64 3E 32 30 30 35 2D 30 32 2D 31 30 54 30 39 3A 34 31 3A 30 30 2E 30 30 30 30 30 30 30 2D 30 35 3A 30 30 3C 2F 55 70 64 61 74 65 64 3E 3C 53 74 61 74 75 73 3E 41 76 61 69 6C 61 62 6C 65 20 4E 65 77 3C 2F 53 74 61 74 75 73 3E 3C 44 65 73 63 72 69 70 74 69 6F 6E " +
				"3E 41 20 63 75 73 74 6F 6D 69 7A 61 62 6C 65 20 4D 65 73 73 61 67 65 42 6F 78 20 77 69 74 68 20 73 6F 75 6E 64 3B 20 53 65 63 74 69 6F 6E 73 3A 20 43 23 20 43 6F 6E 74 72 6F 6C 73 3B 20 53 75 62 73 65 63 74 69 6F 6E 3A 20 55 6E 65 64 69 74 65 64 20 52 65 61 64 65 72 20 43 6F 6E 74 72 69 62 75 74 69 6F 6E 73 29 3C 2F 44 65 73 63 72 69 70 74 69 6F 6E 3E 3C 2F 41 72 74 69 63 6C 65 42 72 69 65 66 3E 3C 41 72 74 69 63 6C 65 42 72 69 65 66 3E 3C 55 52 4C 3E 68 74 74 70 3A 2F 2F 77 77 77 2E 63 6F 64 65 70 72 6F 6A 65 63 74 2E 63 6F 6D 2F 63 73 68 61 72 70 2F 41 72 74 69 63 6C 65 53 75 62 5A 65 72 6F 2E 61 73 70 3C 2F 55 52 4C 3E 3C 54 69 74 6C 65 3E 41 72 74 69 63 6C 65 20 53 75 62 5A 65 72 6F 3A 20 43 61 73 65 42 61 73 65 2C 20 41 20 54 65 73 74 69 6E 67 20 46 72 61 6D 65 77 6F 72 6B 20 66 6F 72 20 43 23 3C 2F 54 69 74 6C 65 3E 3C 41 75 74 68 6F 72 3E 54 6F 6D 20 4F 6C 6C 61 72 2C 20 4A 69 6D 20 42 65 6E 6E 65 74 74 3C 2F 41 75 74 68 6F 72 3E 3C 50 6F 73 74 65 64 3E 32 30 30 35 2D 30 31 2D 32 37 54 " +
				"31 36 3A 32 30 3A 30 30 2E 30 30 30 30 30 30 30 2D 30 35 3A 30 30 3C 2F 50 6F 73 74 65 64 3E 3C 55 70 64 61 74 65 64 3E 32 30 30 35 2D 30 32 2D 31 30 54 30 35 3A 32 33 3A 30 30 2E 30 30 30 30 30 30 30 2D 30 35 3A 30 30 3C 2F 55 70 64 61 74 65 64 3E 3C 53 74 61 74 75 73 3E 41 76 61 69 6C 61 62 6C 65 20 4E 65 77 20 55 70 64 61 74 65 64 20 4D 6F 76 65 64 3C 2F 53 74 61 74 75 73 3E 3C 44 65 73 63 72 69 70 74 69 6F 6E 3E 44 65 73 63 72 69 62 65 73 20 61 20 74 65 73 74 69 6E 67 20 66 72 61 6D 65 77 6F 72 6B 20 66 6F 72 20 75 73 65 20 77 69 74 68 20 56 69 73 75 61 6C 20 53 74 75 64 69 6F 20 61 6E 64 20 43 23 2E 3B 20 53 65 63 74 69 6F 6E 73 3A 20 43 23 20 50 72 6F 67 72 61 6D 6D 69 6E 67 3B 20 53 75 62 73 65 63 74 69 6F 6E 3A 20 44 65 62 75 67 20 61 6E 64 20 54 72 61 63 69 6E 67 29 3C 2F 44 65 73 63 72 69 70 74 69 6F 6E 3E 3C 2F 41 72 74 69 63 6C 65 42 72 69 65 66 3E 3C 41 72 74 69 63 6C 65 42 72 69 65 66 3E 3C 55 52 4C 3E 68 74 74 70 3A 2F 2F 77 77 77 2E 63 6F 64 65 70 72 6F 6A 65 63 74 2E 63 6F 6D " +
				"2F 63 73 2F 6D 69 73 63 63 74 72 6C 2F 52 69 63 68 54 65 78 74 42 6F 78 45 78 74 65 6E 64 65 64 2E 61 73 70 3C 2F 55 52 4C 3E 3C 54 69 74 6C 65 3E 52 69 63 68 54 65 78 74 42 6F 78 45 78 74 65 6E 64 65 64 3C 2F 54 69 74 6C 65 3E 3C 41 75 74 68 6F 72 3E 52 69 63 68 61 72 64 20 50 61 72 73 6F 6E 73 3C 2F 41 75 74 68 6F 72 3E 3C 50 6F 73 74 65 64 3E 32 30 30 35 2D 30 32 2D 31 30 54 30 34 3A 35 32 3A 30 </Bytes>\r\n" +
				"  </Packet>\r\n" +
				"  <Packet Time=\"2005-02-10 20:22:16.36\" ServerFilter=\"209.171.52.99\" PortFilter=\"80\" PacketType=\"Ethernet\">\r\n" +
				"    <Bytes>00 11 F5 0B 1F 69 00 11 F5 0B 1F 6A 08 00 45 00 00 34 DE 90 40 00 80 06 54 78 C0 A8 01 04 D1 AB 34 63 09 E6 00 50 93 B5 51 D1 93 6D 31 22 80 10 FF FF 63 F2 00 00 01 01 05 0A 93 6D 36 AE 93 6D 3C 3A </Bytes>\r\n" +
				"  </Packet>\r\n" +
				"  <Packet Time=\"2005-02-10 20:22:16.41\" ServerFilter=\"209.171.52.99\" PortFilter=\"80\" PacketType=\"Ethernet\">\r\n" +
				"    <Bytes>00 11 F5 0B 1F 6A 00 11 F5 0B 1F 69 08 00 45 00 05 23 6C 0E 40 00 66 06 DC 0B D1 AB 34 63 C0 A8 01 04 00 50 09 E6 93 6D 3C 3A 93 B5 51 D1 50 18 FA 91 DB 6D 00 00 30 2E 30 30 30 30 30 30 30 2D 30 35 3A 30 30 3C 2F 50 6F 73 74 65 64 3E 3C 55 70 64 61 74 65 64 3E 32 30 30 35 2D 30 32 2D 31 30 54 30 34 3A 35 32 3A 30 30 2E 30 30 30 30 30 30 30 2D 30 35 3A 30 30 3C 2F 55 70 64 61 74 65 64 3E 3C 53 74 61 74 75 73 3E 41 76 61 69 6C 61 62 6C 65 20 4E 65 77 3C 2F 53 74 61 74 75 73 3E 3C 44 65 73 63 72 69 70 74 69 6F 6E 3E 41 20 52 69 63 68 54 65 78 74 42 6F 78 20 77 69 74 68 20 61 20 52 69 63 68 54 65 78 74 20 74 6F 6F 6C 62 61 72 2E 3B 20 53 65 63 74 69 6F 6E 73 3A 20 43 23 20 43 6F 6E 74 72 6F 6C 73 3B 20 53 75 62 73 65 63 74 69 6F 6E 3A 20 45 64 69 74 20 43 6F 6E 74 72 6F 6C 73 29 3C 2F 44 65 73 63 72 69 70 74 69 6F 6E 3E 3C 2F 41 72 74 69 63 6C 65 42 72 69 65 66 3E 3C 41 72 74 69 63 6C 65 42 72 69 65 66 3E 3C 55 52 4C 3E 68 74 74 70 3A 2F 2F 77 77 77 2E 63 6F 64 65 70 72 6F 6A 65 63 74 " +
				"2E 63 6F 6D 2F 66 69 6C 65 2F 46 69 6E 64 57 68 65 6E 2E 61 73 70 3C 2F 55 52 4C 3E 3C 54 69 74 6C 65 3E 46 69 6E 64 57 68 65 6E 3C 2F 54 69 74 6C 65 3E 3C 41 75 74 68 6F 72 3E 52 6F 67 65 72 36 35 3C 2F 41 75 74 68 6F 72 3E 3C 50 6F 73 74 65 64 3E 32 30 30 35 2D 30 32 2D 31 30 54 30 33 3A 35 39 3A 30 30 2E 30 30 30 30 30 30 30 2D 30 35 3A 30 30 3C 2F 50 6F 73 74 65 64 3E 3C 55 70 64 61 74 65 64 3E 32 30 30 35 2D 30 32 2D 31 30 54 30 33 3A 35 39 3A 30 30 2E 30 30 30 30 30 30 30 2D 30 35 3A 30 30 3C 2F 55 70 64 61 74 65 64 3E 3C 53 74 61 74 75 73 3E 41 76 61 69 6C 61 62 6C 65 20 4E 65 77 3C 2F 53 74 61 74 75 73 3E 3C 44 65 73 63 72 69 70 74 69 6F 6E 3E 41 6E 20 61 72 74 69 63 6C 65 20 6F 6E 20 66 69 6E 64 69 6E 67 20 66 69 6C 65 73 20 63 72 65 61 74 65 64 20 62 65 66 6F 72 65 20 61 20 64 61 74 65 2E 3B 20 53 65 63 74 69 6F 6E 73 3A 20 46 69 6C 65 73 20 61 6E 64 20 46 6F 6C 64 65 72 73 3B 20 53 75 62 73 65 63 74 69 6F 6E 3A 20 47 65 6E 65 72 61 6C 29 3C 2F 44 65 73 63 72 69 70 74 69 6F 6E 3E 3C " +
				"2F 41 72 74 69 63 6C 65 42 72 69 65 66 3E 3C 41 72 74 69 63 6C 65 42 72 69 65 66 3E 3C 55 52 4C 3E 68 74 74 70 3A 2F 2F 77 77 77 2E 63 6F 64 65 70 72 6F 6A 65 63 74 2E 63 6F 6D 2F 72 69 63 68 65 64 69 74 2F 43 4F 6C 65 52 69 63 68 45 64 69 74 43 74 72 6C 2E 61 73 70 3C 2F 55 52 4C 3E 3C 54 69 74 6C 65 3E 41 20 52 69 63 68 20 45 64 69 74 20 43 6F 6E 74 72 6F 6C 20 54 68 61 74 20 44 69 73 70 6C 61 79 73 20 42 69 74 6D 61 70 73 20 61 6E 64 20 4F 74 68 65 72 20 4F 4C 45 20 4F 62 6A 65 63 74 73 3C 2F 54 69 74 6C 65 3E 3C 41 75 74 68 6F 72 3E 4D 69 6B 65 20 4F 27 4E 65 69 6C 6C 3C 2F 41 75 74 68 6F 72 3E 3C 50 6F 73 74 65 64 3E 32 30 30 35 2D 30 32 2D 31 30 54 30 33 3A 31 31 3A 30 30 2E 30 30 30 30 30 30 30 2D 30 35 3A 30 30 3C 2F 50 6F 73 74 65 64 3E 3C 55 70 64 61 74 65 64 3E 32 30 30 35 2D 30 32 2D 31 30 54 30 33 3A 31 31 3A 30 30 2E 30 30 30 30 30 30 30 2D 30 35 3A 30 30 3C 2F 55 70 64 61 74 65 64 3E 3C 53 74 61 74 75 73 3E 41 76 61 69 6C 61 62 6C 65 20 4E 65 77 3C 2F 53 74 61 74 75 73 3E 3C 44 " +
				"65 73 63 72 69 70 74 69 6F 6E 3E 43 4F 6C 65 52 69 63 68 45 64 69 74 43 74 72 6C 20 77 69 6C 6C 20 64 69 73 70 6C 61 79 20 52 54 46 20 74 65 78 74 20 61 73 20 77 65 6C 6C 20 61 73 20 62 69 74 6D 61 70 73 2C 20 76 69 64 65 6F 20 63 6C 69 70 73 2C 20 57 6F 72 64 2C 20 45 78 63 65 6C 20 61 6E 64 20 50 6F 77 65 72 50 6F 69 6E 74 20 64 6F 63 75 6D 65 6E 74 73 2C 20 61 6E 64 20 61 6E 79 20 6F 74 68 65 72 20 6B 69 6E 64 20 6F 66 20 4F 4C 45 20 6F 62 6A 65 63 74 73 2E 3B 20 53 65 63 74 69 6F 6E 73 3A 20 52 69 63 68 20 45 64 69 74 20 43 6F 6E 74 72 6F 6C 3B 20 53 75 62 73 65 63 74 69 6F 6E 3A 20 47 65 6E 65 72 61 6C 29 3C 2F 44 65 73 63 72 69 70 74 69 6F 6E 3E 3C 2F 41 72 74 69 63 6C 65 42 72 69 65 66 3E 3C 2F 47 65 74 4C 61 74 65 73 74 41 72 74 69 63 6C 65 42 72 69 65 66 52 65 73 75 6C 74 3E 3C 2F 47 65 74 4C 61 74 65 73 74 41 72 74 69 63 6C 65 42 72 69 65 66 52 65 73 70 6F 6E 73 65 3E 3C 2F 73 6F 61 70 3A 42 6F 64 79 3E 3C 2F 73 6F 61 70 3A 45 6E 76 65 6C 6F 70 65 3E </Bytes>\r\n" +
				"  </Packet>\r\n" +
				"  <Packet Time=\"2005-02-10 20:22:16.46\" ServerFilter=\"209.171.52.99\" PortFilter=\"80\" PacketType=\"Ethernet\">\r\n" +
				"    <Bytes>00 11 F5 0B 1F 69 00 11 F5 0B 1F 6A 08 00 45 00 00 34 DE 91 40 00 80 06 54 77 C0 A8 01 04 D1 AB 34 63 09 E6 00 50 93 B5 51 D1 93 6D 31 22 80 10 FF FF 5E F7 00 00 01 01 05 0A 93 6D 36 AE 93 6D 41 35 </Bytes>\r\n" +
				"  </Packet>\r\n" +
				"  <Packet Time=\"2005-02-10 20:22:18.12\" ServerFilter=\"209.171.52.99\" PortFilter=\"80\" PacketType=\"Ethernet\">\r\n" +
				"    <Bytes>00 11 F5 0B 1F 6A 00 11 F5 0B 1F 69 08 00 45 00 05 B4 6D 6D 40 00 66 06 DA 1B D1 AB 34 63 C0 A8 01 04 00 50 09 E6 93 6D 31 22 93 B5 51 D1 50 10 FA 91 ED 64 00 00 20 53 65 63 74 69 6F 6E 73 3A 20 43 2B 2B 20 2F 20 4D 46 43 3B 20 53 75 62 73 65 63 74 69 6F 6E 3A 20 55 6E 65 64 69 74 65 64 20 52 65 61 64 65 72 20 43 6F 6E 74 72 69 62 75 74 69 6F 6E 73 29 3C 2F 44 65 73 63 72 69 70 74 69 6F 6E 3E 3C 2F 41 72 74 69 63 6C 65 42 72 69 65 66 3E 3C 41 72 74 69 63 6C 65 42 72 69 65 66 3E 3C 55 52 4C 3E 68 74 74 70 3A 2F 2F 77 77 77 2E 63 6F 64 65 70 72 6F 6A 65 63 74 2E 63 6F 6D 2F 75 73 65 72 69 74 65 6D 73 2F 6D 61 64 78 6C 69 62 2E 61 73 70 3C 2F 55 52 4C 3E 3C 54 69 74 6C 65 3E 44 4C 4C 20 46 6F 72 20 42 75 66 66 65 72 2D 62 61 73 65 64 20 4D 50 33 20 44 65 63 6F 64 69 6E 67 3C 2F 54 69 74 6C 65 3E 3C 41 75 74 68 6F 72 3E 67 69 6C 61 64 2D 61 70 3C 2F 41 75 74 68 6F 72 3E 3C 50 6F 73 74 65 64 3E 32 30 30 35 2D 30 32 2D 30 34 54 31 32 3A 35 37 3A 30 30 2E 30 30 30 30 30 30 30 2D 30 35 3A " +
				"30 30 3C 2F 50 6F 73 74 65 64 3E 3C 55 70 64 61 74 65 64 3E 32 30 30 35 2D 30 32 2D 31 30 54 31 30 3A 35 35 3A 30 30 2E 30 30 30 30 30 30 30 2D 30 35 3A 30 30 3C 2F 55 70 64 61 74 65 64 3E 3C 53 74 61 74 75 73 3E 41 76 61 69 6C 61 62 6C 65 20 4E 65 77 20 55 70 64 61 74 65 64 3C 2F 53 74 61 74 75 73 3E 3C 44 65 73 63 72 69 70 74 69 6F 6E 3E 53 6F 75 72 63 65 20 63 6F 64 65 20 74 6F 20 70 72 6F 64 75 63 65 20 61 20 44 4C 4C 20 74 68 61 74 20 63 6F 6E 76 65 72 74 73 20 4D 50 33 20 66 69 6C 65 73 20 74 6F 20 50 43 4D 20 75 73 69 6E 67 20 61 20 62 75 66 66 65 72 2D 62 61 73 65 64 20 6D 65 63 68 61 6E 69 73 6D 2E 20 42 61 73 65 64 20 6F 6E 20 74 68 65 20 6F 70 65 6E 20 73 6F 75 72 63 65 20 6C 69 62 72 61 72 79 20 6C 69 62 6D 61 64 2E 3B 20 53 65 63 74 69 6F 6E 73 3A 20 41 75 64 69 6F 20 61 6E 64 20 56 69 64 65 6F 3B 20 53 75 62 73 65 63 74 69 6F 6E 3A 20 55 6E 65 64 69 74 65 64 20 52 65 61 64 65 72 20 43 6F 6E 74 72 69 62 75 74 69 6F 6E 73 29 3C 2F 44 65 73 63 72 69 70 74 69 6F 6E 3E 3C 2F 41 72 74 " +
				"69 63 6C 65 42 72 69 65 66 3E 3C 41 72 74 69 63 6C 65 42 72 69 65 66 3E 3C 55 52 4C 3E 68 74 74 70 3A 2F 2F 77 77 77 2E 63 6F 64 65 70 72 6F 6A 65 63 74 2E 63 6F 6D 2F 75 73 65 72 69 74 65 6D 73 2F 47 65 74 4C 6F 67 67 69 6E 67 57 69 74 68 45 6E 74 4C 69 62 2E 61 73 70 3C 2F 55 52 4C 3E 3C 54 69 74 6C 65 3E 47 65 74 20 4C 6F 67 67 69 6E 67 20 77 69 74 68 20 74 68 65 20 45 6E 74 65 72 70 72 69 73 65 20 4C 69 62 72 61 72 79 3C 2F 54 69 74 6C 65 3E 3C 41 75 74 68 6F 72 3E 50 69 65 72 73 20 4C 61 77 73 6F 6E 3C 2F 41 75 74 68 6F 72 3E 3C 50 6F 73 74 65 64 3E 32 30 30 35 2D 30 32 2D 31 30 54 31 30 3A 30 36 3A 30 30 2E 30 30 30 30 30 30 30 2D 30 35 3A 30 30 3C 2F 50 6F 73 74 65 64 3E 3C 55 70 64 61 74 65 64 3E 32 30 30 35 2D 30 32 2D 31 30 54 31 30 3A 30 36 3A 30 30 2E 30 30 30 30 30 30 30 2D 30 35 3A 30 30 3C 2F 55 70 64 61 74 65 64 3E 3C 53 74 61 74 75 73 3E 41 76 61 69 6C 61 62 6C 65 20 4E 65 77 3C 2F 53 74 61 74 75 73 3E 3C 44 65 73 63 72 69 70 74 69 6F 6E 3E 48 6F 77 20 74 6F 20 67 65 74 20 74 " +
				"68 65 20 62 61 73 69 63 73 20 6F 66 20 74 68 65 20 45 6E 74 65 72 70 72 69 73 65 20 4C 69 62 72 61 72 79 27 73 20 4C 6F 67 67 69 6E 67 20 61 6E 64 20 49 6E 73 74 72 75 6D 65 6E 74 61 74 69 6F 6E 20 41 70 70 6C 69 63 61 74 69 6F 6E 20 42 6C 6F 63 6B 20 77 6F 72 6B 69 6E 67 20 77 69 74 68 20 79 6F 75 72 20 61 70 70 6C 69 63 61 74 69 6F 6E 2E 3B 20 53 65 63 74 69 6F 6E 73 3A 20 2E 4E 45 54 3B 20 53 75 62 73 65 63 74 69 6F 6E 3A 20 55 6E 65 64 69 74 65 64 20 52 65 61 64 65 72 20 43 6F 6E 74 72 69 62 75 74 69 6F 6E 73 29 3C 2F 44 65 73 63 72 69 70 74 69 6F 6E 3E 3C 2F 41 72 74 69 63 6C 65 42 72 69 65 66 3E 3C 41 72 74 69 63 6C 65 42 72 69 65 66 3E 3C 55 52 4C 3E 68 74 74 70 3A 2F 2F 77 77 77 2E 63 6F 64 65 70 72 6F 6A 65 63 74 2E 63 6F 6D 2F 75 73 65 72 69 74 65 6D 73 2F 49 6E 63 72 65 6D 65 6E 74 61 6C 5F 73 65 61 72 63 68 2E 61 73 70 3C 2F 55 52 4C 3E 3C 54 69 74 6C 65 3E 48 6F 77 20 74 6F 20 69 6D 70 6C 65 6D 65 6E 74 73 20 61 6E 20 69 6E 63 72 65 6D 65 6E 74 61 6C 20 73 65 61 72 63 68 20 75 73 " +
				"69 6E 67 20 61 20 67 72 69 64 20 28 55 6C 74 72 61 57 69 6E 47 72 69 64 29 3C 2F 54 69 74 6C 65 3E 3C 41 75 74 68 6F 72 3E 43 46 51 C3 BC 65 62 3C 2F 41 75 74 68 6F 72 3E 3C 50 6F 73 74 65 64 3E 32 30 30 35 2D 30 32 2D 31 30 54 30 39 3A 34 37 3A 30 30 2E 30 30 30 30 30 30 30 2D 30 35 3A 30 30 3C 2F 50 6F 73 74 65 64 3E 3C 55 70 64 61 74 65 64 3E 32 30 30 35 2D 30 32 2D 31 30 54 30 39 3A 34 37 3A 30 </Bytes>\r\n" +
				"  </Packet>\r\n" +
				"  <Packet Time=\"2005-02-10 20:22:18.17\" ServerFilter=\"209.171.52.99\" PortFilter=\"80\" PacketType=\"Ethernet\">\r\n" +
				"    <Bytes>00 11 F5 0B 1F 69 00 11 F5 0B 1F 6A 08 00 45 00 00 28 DE 92 40 00 80 06 54 82 C0 A8 01 04 D1 AB 34 63 09 E6 00 50 93 B5 51 D1 93 6D 41 35 50 10 FF FF 23 BA 00 00 </Bytes>\r\n" +
				"  </Packet>\r\n" +
				"</Packets>";
		}
	}
}
