//
// SoapTrace.Net - A soap message monitoring utility. 
//
// Copyright (C) 2005 Matthew Ward
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Matthew Ward (mrward@users.sourceforge.net)


using System;
using System.Collections;
using System.Net.Sockets;

namespace SoapTrace
{
	/// <summary>
	///   A collection that stores <see cref='MonitorManager'/> objects.
	/// </summary>
	[Serializable()]
	public class MonitorManagerCollection : CollectionBase {
		
		/// <summary>
		///   Initializes a new instance of <see cref='MonitorManagerCollection'/>.
		/// </summary>
		public MonitorManagerCollection()
		{
		}
		
		/// <summary>
		///   Initializes a new instance of <see cref='MonitorManagerCollection'/> based on another <see cref='MonitorManagerCollection'/>.
		/// </summary>
		/// <param name='val'>
		///   A <see cref='MonitorManagerCollection'/> from which the contents are copied
		/// </param>
		public MonitorManagerCollection(MonitorManagerCollection val)
		{
			this.AddRange(val);
		}
		
		/// <summary>
		///   Initializes a new instance of <see cref='MonitorManagerCollection'/> containing any array of <see cref='MonitorManager'/> objects.
		/// </summary>
		/// <param name='val'>
		///       A array of <see cref='MonitorManager'/> objects with which to intialize the collection
		/// </param>
		public MonitorManagerCollection(MonitorManager[] val)
		{
			this.AddRange(val);
		}
		
		/// <summary>
		///   Represents the entry at the specified index of the <see cref='MonitorManager'/>.
		/// </summary>
		/// <param name='index'>The zero-based index of the entry to locate in the collection.</param>
		/// <value>The entry at the specified index of the collection.</value>
		/// <exception cref='ArgumentOutOfRangeException'><paramref name='index'/> is outside the valid range of indexes for the collection.</exception>
		public MonitorManager this[int index] {
			get {
				return ((MonitorManager)(List[index]));
			}
			set {
				List[index] = value;
			}
		}
		
		/// <summary>
		/// Shuts down all monitors.
		/// </summary>
		public void Shutdown()
		{
			foreach(MonitorManager Manager in this)
			{
				try
				{
					Manager.Stop();
				}
				catch(SocketException)
				{
					// Ignore.	
				}
			}
		}
		
		/// <summary>
		///   Adds a <see cref='MonitorManager'/> with the specified value to the 
		///   <see cref='MonitorManagerCollection'/>.
		/// </summary>
		/// <param name='val'>The <see cref='MonitorManager'/> to add.</param>
		/// <returns>The index at which the new element was inserted.</returns>
		/// <seealso cref='MonitorManagerCollection.AddRange'/>
		public int Add(MonitorManager val)
		{
			return List.Add(val);
		}
		
		/// <summary>
		///   Copies the elements of an array to the end of the <see cref='MonitorManagerCollection'/>.
		/// </summary>
		/// <param name='val'>
		///    An array of type <see cref='MonitorManager'/> containing the objects to add to the collection.
		/// </param>
		/// <seealso cref='MonitorManagerCollection.Add'/>
		public void AddRange(MonitorManager[] val)
		{
			for (int i = 0; i < val.Length; i++) {
				this.Add(val[i]);
			}
		}
		
		/// <summary>
		///   Adds the contents of another <see cref='MonitorManagerCollection'/> to the end of the collection.
		/// </summary>
		/// <param name='val'>
		///    A <see cref='MonitorManagerCollection'/> containing the objects to add to the collection.
		/// </param>
		/// <seealso cref='MonitorManagerCollection.Add'/>
		public void AddRange(MonitorManagerCollection val)
		{
			for (int i = 0; i < val.Count; i++)
			{
				this.Add(val[i]);
			}
		}
		
		/// <summary>
		///   Gets a value indicating whether the 
		///    <see cref='MonitorManagerCollection'/> contains the specified <see cref='MonitorManager'/>.
		/// </summary>
		/// <param name='val'>The <see cref='MonitorManager'/> to locate.</param>
		/// <returns>
		/// <see langword='true'/> if the <see cref='MonitorManager'/> is contained in the collection; 
		///   otherwise, <see langword='false'/>.
		/// </returns>
		/// <seealso cref='MonitorManagerCollection.IndexOf'/>
		public bool Contains(MonitorManager val)
		{
			return List.Contains(val);
		}
		
		/// <summary>
		///   Copies the <see cref='MonitorManagerCollection'/> values to a one-dimensional <see cref='Array'/> instance at the 
		///    specified index.
		/// </summary>
		/// <param name='array'>The one-dimensional <see cref='Array'/> that is the destination of the values copied from <see cref='MonitorManagerCollection'/>.</param>
		/// <param name='index'>The index in <paramref name='array'/> where copying begins.</param>
		/// <exception cref='ArgumentException'>
		///   <para><paramref name='array'/> is multidimensional.</para>
		///   <para>-or-</para>
		///   <para>The number of elements in the <see cref='MonitorManagerCollection'/> is greater than
		///         the available space between <paramref name='arrayIndex'/> and the end of
		///         <paramref name='array'/>.</para>
		/// </exception>
		/// <exception cref='ArgumentNullException'><paramref name='array'/> is <see langword='null'/>. </exception>
		/// <exception cref='ArgumentOutOfRangeException'><paramref name='arrayIndex'/> is less than <paramref name='array'/>'s lowbound. </exception>
		/// <seealso cref='Array'/>
		public void CopyTo(MonitorManager[] array, int index)
		{
			List.CopyTo(array, index);
		}
		
		/// <summary>
		///    Returns the index of a <see cref='MonitorManager'/> in 
		///       the <see cref='MonitorManagerCollection'/>.
		/// </summary>
		/// <param name='val'>The <see cref='MonitorManager'/> to locate.</param>
		/// <returns>
		///   The index of the <see cref='MonitorManager'/> of <paramref name='val'/> in the 
		///   <see cref='MonitorManagerCollection'/>, if found; otherwise, -1.
		/// </returns>
		/// <seealso cref='MonitorManagerCollection.Contains'/>
		public int IndexOf(MonitorManager val)
		{
			return List.IndexOf(val);
		}
		
		/// <summary>
		///   Inserts a <see cref='MonitorManager'/> into the <see cref='MonitorManagerCollection'/> at the specified index.
		/// </summary>
		/// <param name='index'>The zero-based index where <paramref name='val'/> should be inserted.</param>
		/// <param name='val'>The <see cref='MonitorManager'/> to insert.</param>
		/// <seealso cref='MonitorManagerCollection.Add'/>
		public void Insert(int index, MonitorManager val)
		{
			List.Insert(index, val);
		}
		
		/// <summary>
		///  Returns an enumerator that can iterate through the <see cref='MonitorManagerCollection'/>.
		/// </summary>
		/// <seealso cref='IEnumerator'/>
		public new MonitorManagerEnumerator GetEnumerator()
		{
			return new MonitorManagerEnumerator(this);
		}
		
		/// <summary>
		///   Removes a specific <see cref='MonitorManager'/> from the <see cref='MonitorManagerCollection'/>.
		/// </summary>
		/// <param name='val'>The <see cref='MonitorManager'/> to remove from the <see cref='MonitorManagerCollection'/>.</param>
		/// <exception cref='ArgumentException'><paramref name='val'/> is not found in the Collection.</exception>
		public void Remove(MonitorManager val)
		{
			List.Remove(val);
		}
		
		/// <summary>
		///   Enumerator that can iterate through a MonitorManagerCollection.
		/// </summary>
		/// <seealso cref='IEnumerator'/>
		/// <seealso cref='MonitorManagerCollection'/>
		/// <seealso cref='MonitorManager'/>
		public class MonitorManagerEnumerator : IEnumerator
		{
			IEnumerator baseEnumerator;
			IEnumerable temp;
			
			/// <summary>
			///   Initializes a new instance of <see cref='MonitorManagerEnumerator'/>.
			/// </summary>
			public MonitorManagerEnumerator(MonitorManagerCollection mappings)
			{
				this.temp = ((IEnumerable)(mappings));
				this.baseEnumerator = temp.GetEnumerator();
			}
			
			/// <summary>
			///   Gets the current <see cref='MonitorManager'/> in the <seealso cref='MonitorManagerCollection'/>.
			/// </summary>
			public MonitorManager Current {
				get {
					return ((MonitorManager)(baseEnumerator.Current));
				}
			}
			
			object IEnumerator.Current {
				get {
					return baseEnumerator.Current;
				}
			}
			
			/// <summary>
			///   Advances the enumerator to the next <see cref='MonitorManager'/> of the <see cref='MonitorManagerCollection'/>.
			/// </summary>
			public bool MoveNext()
			{
				return baseEnumerator.MoveNext();
			}
			
			/// <summary>
			///   Sets the enumerator to its initial position, which is before the first element in the <see cref='MonitorManagerCollection'/>.
			/// </summary>
			public void Reset()
			{
				baseEnumerator.Reset();
			}
		}
	}
}
